#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 18 12:02:31 2021

@author: opid02
"""

import argparse
from PyQt5.QtWidgets import QApplication
from xpcsutilities.gui.mainwindow import MainWindow
import logging
import sys

logging.basicConfig(format='%(levelname)s %(name)s:%(lineno)d %(message)s',
                    handlers=[logging.StreamHandler(sys.stdout)])
logger = logging.getLogger(__name__)

def main():
    
    parser = argparse.ArgumentParser(description="Display XPCS result files")
    
    parser.add_argument('-v', dest='verbose', action='store_true', default=False, help='Verbose mode (minimum message level=debug)')
    parser.add_argument('--quiet', dest='quiet', action='store_true', default=False, help='Quiet mode (minimum message level=warning)')
    
    args = parser.parse_args()
    
    print(args)
            
    # Set the log level of each package according to the defined verbosity
    loglevel = logging.INFO
    
    if args.verbose:
        loglevel = logging.DEBUG
    elif args.quiet:
        loglevel = logging.WARNING
        
    logger.setLevel(loglevel)
    
        
    for name in logging.root.manager.loggerDict:
        if name.startswith('xpcsutilities'):
            logger.debug(name)
            logging.getLogger(name).setLevel(loglevel)
            
    
    app = QApplication([])
    
    win = MainWindow()
    
    sys.exit(app.exec_())


# -*- coding: utf-8 -*-



import matplotlib
matplotlib.use('Qt5Agg')

import PyQt5.QtWidgets as qtw

import numpy as np
import scipy.optimize as opt

import traceback
import logging
logger = logging.getLogger(__name__)

from .datafilewidget import DataFileModel, DataFileWidget
import xpcsutilities.tools.graphs as gt
from .resultgraph import ResultFitGraph, ResultParamsFitGraph
from .curvefitwidget import CurveFitWidget

class FitTab(qtw.QWidget):
    
    OPT_COLOR_Q     = 0
    OPT_COLOR_QIDX  = 1
    OPT_COLOR_TIME  = 2
    
    def __init__(self):
        super().__init__()
        
        #Layout
        self.__layout = qtw.QGridLayout()
        self.__layout.setColumnStretch(0,1)
        self.__layout.setColumnStretch(1,3)
        self.setLayout(self.__layout)
        
        # File list
        self.__dataList = DataFileWidget()
        self.__layout.addWidget(self.__dataList, 0,0)
        
        # Contrast rescaling
        self.__con_rescale = qtw.QGroupBox("Rescale contrast before fitting")
        self.__layout.addWidget(self.__con_rescale,1,0)
        self.__con_rescale.setCheckable(True)
        self.__con_rescale.setChecked(True)
        self.__con_rescale.toggled.connect(self.__onContrastRescaleChanged)
        
        self.__con_rescale_layout = qtw.QFormLayout()
        self.__con_rescale.setLayout(self.__con_rescale_layout)
        
        self.__con_rescale_method = qtw.QComboBox()
        self.__con_rescale_layout.addRow("Rescaling method", self.__con_rescale_method)
        
        self.__con_rescale_method.addItem("Average")
        self.__con_rescale_method.addItem("Fit linear")
        self.__con_rescale_method.addItem("Fit exponential")
        
        self.__con_rescale_method.setCurrentIndex(1)
        
        self.__con_rescale_limit = qtw.QDoubleSpinBox()
        self.__con_rescale_layout.addRow("Maximum lag time", self.__con_rescale_limit)
        self.__con_rescale_limit.setDecimals(6)
        self.__con_rescale_limit.setMaximum(np.inf)
        self.__con_rescale_limit.setValue(0.0035)
        
        # Curve fitting
        self.__curve_fit_widget = CurveFitWidget()
        self.__layout.addWidget(self.__curve_fit_widget,2,0)
        self.__curve_fit_widget.setCheckable(True)
        self.__curve_fit_widget.setChecked(True)
                        
        # Results         
        self.__tabreswidget = qtw.QTabWidget()
        self.__tabreswidget.setTabsClosable(True)
        self.__tabreswidget.tabCloseRequested.connect(self.__onTabCloseRequested)
        self.__layout.addWidget(self.__tabreswidget,0,1,4,1)
        
        w = qtw.QWidget()
        self.__reslayout = qtw.QVBoxLayout()
        w.setLayout(self.__reslayout)
        self.__tabreswidget.addTab(w, "Results" )
        
        self.__tabrestable = qtw.QTableWidget()
        header = self.__tabrestable.horizontalHeader()
        header.setSectionResizeMode(qtw.QHeaderView.ResizeToContents)
        self.__reslayout.addWidget(self.__tabrestable)
        
        self.__optlayout = qtw.QHBoxLayout()
        self.__reslayout.addLayout(self.__optlayout)
        
        self.__opt1layout = qtw.QFormLayout()
        self.__optlayout.addLayout(self.__opt1layout)
        
        self.__optgroupbyfile = qtw.QCheckBox()
        self.__optgroupbyfile.setChecked(True)
        self.__opt1layout.addRow("Separate each files", self.__optgroupbyfile)
        
        self.__optgroupbydir = qtw.QCheckBox()
        self.__optgroupbydir.setChecked(True)
        self.__opt1layout.addRow("Separate each direction", self.__optgroupbydir)
        
        self.__optgroupbyqid = qtw.QCheckBox()
        self.__optgroupbyqid.setChecked(False)
        self.__opt1layout.addRow("Separate each q-index", self.__optgroupbyqid)
        
        self.__optcolorby = qtw.QComboBox()
        self.__opt1layout.addRow("Color", self.__optcolorby)
        
        self.__optcolorby.insertItem(self.OPT_COLOR_Q, 'q')
        self.__optcolorby.insertItem(self.OPT_COLOR_QIDX, 'q index')
        self.__optcolorby.insertItem(self.OPT_COLOR_TIME, 'time')
        
        self.__fitsavebtn = qtw.QPushButton("Save results")
        self.__fitsavebtn.setMinimumHeight(100)
        self.__optlayout.addWidget(self.__fitsavebtn)
        self.__fitsavebtn.clicked.connect(self.savefit)
        
        self.__fitdisplaybtn = qtw.QPushButton("Display selected results")
        self.__fitdisplaybtn.setMinimumHeight(100)
        self.__optlayout.addWidget(self.__fitdisplaybtn)
        self.__fitdisplaybtn.clicked.connect(self.displayfit)
        
        self.__fitdisplayparamsbtn = qtw.QPushButton("Display parameters")
        self.__fitdisplayparamsbtn.setMinimumHeight(100)
        self.__optlayout.addWidget(self.__fitdisplayparamsbtn)
        self.__fitdisplayparamsbtn.clicked.connect(self.displayfitparams)
        
        # Button
        self.__btnlayout = qtw.QHBoxLayout()
        self.__layout.addLayout(self.__btnlayout,3,0)
        
        self.__inibtn = qtw.QPushButton("Display  curves with initial values")
        self.__inibtn.setMinimumHeight(100)
        self.__btnlayout.addWidget(self.__inibtn)
        self.__inibtn.clicked.connect(self.initdisplay)
        
        self.__fitbtn = qtw.QPushButton("Fit")
        self.__fitbtn.setMinimumHeight(100)
        self.__btnlayout.addWidget(self.__fitbtn)
        self.__fitbtn.clicked.connect(self.fit)
        
    def __onTabCloseRequested(self, idx : int):
        if idx == 0:
            return
        
        self.__tabreswidget.removeTab(idx)
        
    def __onContrastRescaleChanged(self):
        self.__curve_fit_widget.setFitContrast(not self.__con_rescale.isChecked())
        
    def clear(self):
        self.__dataList.clearFiles()
        
    def plot(self, files):
        logger.debug(files)
        self.__dataList.loadFiles(files)
                
    def initdisplay(self):
        
        fnc, parval = self.__curve_fit_widget.fitfunction()
        pars = self.__curve_fit_widget.args2params(1,parval, *self.__curve_fit_widget.initValues(1))
        
        logger.debug(parval)
        logger.debug(pars)
        
        sepfile = self.__optgroupbyfile.isChecked()
        sepdir = self.__optgroupbydir.isChecked()
        sepqid = self.__optgroupbyqid.isChecked()
        
        resd = dict()
        
        minc = +np.inf
        maxc = -np.inf
        
        mints = +np.inf
                        
        seldata, NROWS = self.__dataList.getSelectedDatas()
            
        for d in seldata:
            for i, qid in enumerate(d['q-index']):
                key = ""
                if sepfile:
                    key += "__fn__"+d['filename']
                if sepdir:
                    key += "__dir__"+d['direction']
                if sepqid:
                    key += "__qid__%i"%qid
                    
                if key not in resd:
                    resd[key] = []
                    
                if self.__optcolorby.currentIndex() == self.OPT_COLOR_Q:
                    colv = d['qs'][i]
                elif self.__optcolorby.currentIndex() == self.OPT_COLOR_QIDX:
                    colv = qid
                elif self.__optcolorby.currentIndex() == self.OPT_COLOR_TIME:
                    colv = d['ts'] - mints
                
                if colv > maxc: maxc = colv
                if colv < minc: minc = colv
                    
                resd[key] += [{'filename': d['filename'],
                               'direction': d['direction'],
                               'ts': d['ts'],
                               'tsr': d['tsr'],
                               'q-index': qid,
                               'color': colv,
                               'q': d['qs'][i],
                               'lag': d['lag'].ravel(),
                               'cf': d['cf'][i,:],
                               'fitres': pars.copy(),
                               'fitkeys': pars.keys(),
                               'fitfun': fnc},]
    
        (cmapi, normi, mv) = gt.qindex_colors([minc, maxc])
    
        try:
            # Get contrast
            if self.__con_rescale.isChecked():
                for k,r in resd.items():
                    for l in r:
                        lag = l['lag']
                        msk = lag < self.__con_rescale_limit.value()
                        lag = lag[msk]
                        cf = l['cf'][msk]
                        
                        beta = None
                        
                        if self.__con_rescale_method.currentIndex() == 0: # Average
                            beta = np.mean(cf)-1.
                        elif self.__con_rescale_method.currentIndex() == 1: # Linear fit
                            k = np.polyfit(lag, cf, 1)
                            beta = k[1]-1.
                        elif self.__con_rescale_method.currentIndex() == 2: # Exp fit
                            fun = lambda lt, k0, tau: k0*np.exp(-lt/tau)+1.
                            k = opt.curve_fit(fun, lag, cf, [ 0.4, 1e-2 ], bounds=([ 0, 0 ], [ 1, +np.inf ]))
                            beta = k[0][0]
                        l['fitres']['beta'] = beta
            
            #print(resd)
            
            i=0
            for k,res in resd.items():
                fitw = ResultFitGraph(res, cmapi, normi)
                self.__tabreswidget.addTab(fitw, "Init %i"%i)
                i += 1
            
            self.__tabreswidget.setCurrentIndex(self.__tabreswidget.count()-1)
        except Exception as e:
            logger.fatal(traceback.format_exc())
            qtw.QMessageBox.critical(self, "Error", str(e))
                
    def fit(self):
        try:
            self.__resdata, NROWS = self.__dataList.getSelectedDatas()
                                    
            if len(self.__resdata) == 0:
                logger.warning("No data to fit")
                return
            
            # Contrast
            if self.__con_rescale.isChecked():
                for i,l in enumerate(self.__resdata):
                    lag = l['lag'].ravel()
                    msk = lag < self.__con_rescale_limit.value()
                    
                    beta = np.empty((len(l['q-index']),), np.float32)
                    
                    for j, qid in enumerate(l['q-index']):
                        if self.__con_rescale_method.currentIndex() == 0: # Average
                            beta[j] = np.mean(l['cf'][j,msk])-1.
                        elif self.__con_rescale_method.currentIndex() == 1: # Linear fit
                            
                            logger.debug((l['cf'].shape, j, msk.shape))
                            
                            cf = l['cf'][j,msk]
                            
                            k = np.polyfit(lag[msk], cf, 1)
                            beta[j] = k[1]-1.
                        elif self.__con_rescale_method.currentIndex() == 2: # Exp fit
                            cf = l['cf'][j,msk]
                            k = opt.curve_fit(lambda lt, k0, tau: k0*np.exp(-lt/tau)+1., lag[msk], cf, [ 0.4, 1e-2 ], bounds=([ 0, 0 ], [ 1, +np.inf ]))
                            beta[j] = k[0][0]
                    
                    l['fitres']['beta'] = beta 
                        
            # Curve fitting
            if self.__curve_fit_widget.isChecked():
                for i,l in enumerate(self.__resdata):
                    fun, parv = self.__curve_fit_widget.fitfunction()
                    keys = parv.keys()
                    
                    fitfnc = lambda t, *args: fun(t, np.array(l['qs']), *args).ravel()
                    
                    l['fitfun'] = fun
                    l['fitkeys'] = keys
                    
                    msk = l['lag'] < self.__curve_fit_widget.maxlag()
                    
                    if self.__con_rescale.isChecked():
                        cf =  (l['cf'].T-1.)/l['fitres']['beta']
                    else:
                        cf = l['cf'].T-1.
                    
                    n = len(l['qs'])
                    
                    # WARNING: keep initValues to 1 because the scaling is done in args2params according to initial values
                    
                    initv = self.__curve_fit_widget.initValues(n)
                    bounds = self.__curve_fit_widget.bounds(n)
                    
                    logger.debug("Fit initial value: "+str(initv))
                    logger.debug("Fit bounds: "+str(bounds))
                    
                    msk[0] = False
                    
                    msk = np.logical_and(msk, np.sum(~np.isfinite(cf), axis=1) == 0)
                    
                    kopt = opt.curve_fit(fitfnc,
                                         l['lag'][msk], cf[msk,:].ravel(),
                                         initv,
                                         bounds=bounds,
                                         method='dogbox',
                                         verbose=0,
                                         loss='linear', # linear ( default, soft_1l, cauchy )
                                         ftol=1e-12,
                                         gtol=1e-12,
                                         xtol=1e-12,
                                         )
                    
                    logger.debug(kopt[1])
                    logger.debug(parv)
                    logger.debug(n)
                    
                    params = self.__curve_fit_widget.args2params(n, parv, *kopt[0])
                    
                    logger.debug(params)
                    
                    for k,v in params.items():
                        l['fitres'][k] = v
                        
                    
        except Exception as e:
            logger.fatal(traceback.format_exc())
            qtw.QMessageBox.critical(self, "Error", str(e))
                    
        # Display results to table
        self.__tabrestable.setColumnCount(5+len( self.__resdata[0]['fitres'].keys()))
        self.__tabrestable.setRowCount(NROWS)
        self.__tabrestable.setHorizontalHeaderLabels(['','filename','direction','q-index','q', *self.__resdata[0]['fitres'].keys()])
        
        self.__tabreswidget.setCurrentIndex(0)
        
        i=0
        for l in self.__resdata:
            l['check'] = []
            for m in range(len(l['q-index'])):
                chkbx = qtw.QCheckBox()
                chkbx.setChecked(True)
                l['check'] += [chkbx, ]
                self.__tabrestable.setCellWidget(i,0,chkbx)
                self.__tabrestable.setItem(i,1,qtw.QTableWidgetItem(l['filename']))
                self.__tabrestable.setItem(i,2,qtw.QTableWidgetItem(l['direction']))
                self.__tabrestable.setItem(i,3,qtw.QTableWidgetItem("%i"%l['q-index'][m]))
                self.__tabrestable.setItem(i,4,qtw.QTableWidgetItem("%.3e"%l['qs'][m]))
                
                for j,k in enumerate(l['fitres'].keys()):
                    self.__tabrestable.setItem(i,j+5,qtw.QTableWidgetItem('%.3e'%l['fitres'][k][m]))
                    
                i += 1
                
    @property
    def strModel(self):
        """
        Return the current model as string
        """
        return "1+beta*(%s)"%self.__curve_fit_widget.strModel()
                
    def savefit(self):
        """
        Save the results in table to csv file
        """
        
        outf = qtw.QFileDialog.getSaveFileName(self, "Save file to", "results.csv")
        outf = outf[0]
        
        if outf == '':
            return
        
        with open(outf, 'w') as fd:
            
            fd.write("# model; g1 = %s\n"%(self.strModel))
            fd.write("## filename;direction;q-index;q (A^-1);%s\n"%(';'.join(self.__resdata[0]['fitres'].keys())))
            
            for l in self.__resdata:
                for m in range(len(l['q-index'])):
                    fd.write(l['filename']+';')
                    fd.write(l['direction']+';')
                    fd.write("%i"%l['q-index'][m]+';')
                    fd.write("%.3e"%l['qs'][m]+';')
                    
                    for j,k in enumerate(l['fitres'].keys()):
                        fd.write('%.3e'%l['fitres'][k][m]+';')
                        
                    fd.write('\n')
                    
    
    def displayfit(self):
        """
        Display the correlation function curve together with fit curve
        """
        
        sepfile = self.__optgroupbyfile.isChecked()
        sepdir = self.__optgroupbydir.isChecked()
        sepqid = self.__optgroupbyqid.isChecked()
        
        resd = dict()
        
        minc = +np.inf
        maxc = -np.inf
        
        for r in self.__resdata:
            for i,qid in enumerate(r['q-index']):
                if not r['check'][i].isChecked():
                    continue
                
                key = ""
                if sepfile:
                    key += "__fn__"+r['filename']
                if sepdir:
                    key += "__dir__"+r['direction']
                if sepqid:
                    key += "__qid__%i"%qid
                    
                if key not in resd:
                    resd[key] = []
                    
                
                                    
                if self.__optcolorby.currentIndex() == self.OPT_COLOR_Q:
                    colv = r['qs'][i]
                elif self.__optcolorby.currentIndex() == self.OPT_COLOR_QIDX:
                    colv = r['q-index'][i]
                elif self.__optcolorby.currentIndex() == self.OPT_COLOR_TIME:
                    colv = r['tsr']
                                
                if colv > maxc: maxc = colv
                if colv < minc: minc = colv
                    
                rl = r.copy()
                del rl['check']
                rl['color'] = colv
                rl['q-index'] = r['q-index'][i]
                rl['q'] = r['qs'][i]
                rl['cf'] = rl['cf'][i,:]
                
                rl['fitres'] = dict()
                for k in r['fitres'].keys():
                    rl['fitres'][k] = r['fitres'][k][i]
                    
                resd[key] += [rl,]
        
        (cmapi, normi, mv) = gt.qindex_colors([minc, maxc])
        i=0
        for k,res in resd.items():
            fitw = ResultFitGraph(res, cmapi, normi)
            self.__tabreswidget.addTab(fitw, "Graph %i"%i)
            i += 1
            
        self.__tabreswidget.setCurrentIndex(self.__tabreswidget.count()-1)
        
    def displayfitparams(self):
        """
        Display the correlation function curve together with fit curve
        """
        
        sepfile = self.__optgroupbyfile.isChecked()
        sepdir = self.__optgroupbydir.isChecked()
        sepqid = self.__optgroupbyqid.isChecked()
        
        resd = dict()
        
        for r in self.__resdata:
            for i,qid in enumerate(r['q-index']):
                if not r['check'][i].isChecked():
                    continue
                
                key = ""
                if sepfile:
                    key += "__fn__"+r['filename']
                if sepdir:
                    key += "__dir__"+r['direction']
                if sepqid:
                    key += "__qid__%i"%qid
                    
                if key not in resd:               
                    resd[key] = {'q'        : [],
                                 'tsr'      : [],
                                 'dir'      : [],
                                 'file'     : [],
                                 'params'   : {k:[] for k in r['fitres']} }
                    
                resd[key]['q'] += [ r['qs'][i], ]
                resd[key]['tsr'] += [ r['tsr'], ]
                resd[key]['dir'] += [ r['direction'], ]
                resd[key]['file'] += [ r['filename'], ]
                
                for k in r['fitres']:
                    resd[key]['params'][k] += [ r['fitres'][k][i], ]
        
        i=0
        for k,res in resd.items():
            fitw = ResultParamsFitGraph(res)
            self.__tabreswidget.addTab(fitw, "Params %i"%i)
            i += 1
            
        self.__tabreswidget.setCurrentIndex(self.__tabreswidget.count()-1)






import matplotlib
matplotlib.use('Qt5Agg')

import PyQt5.QtWidgets as qtw
import PyQt5.QtCore as qtc

import xpcsutilities.tools.graphs as gt
from xpcsutilities.tools.result_file import XPCSResultFile
import numpy as np
import scipy.optimize as opt

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas, NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

import xpcsutilities.tools.common as cmm

import logging
logger = logging.getLogger(__name__) 
        
                 
class Graph2tCorrelation(qtw.QWidget):
    def __init__(self):
        super().__init__()
        
        #Layout
        self.__layout = qtw.QVBoxLayout()
        self.setLayout(self.__layout)
        
        #Figure
        self.__canvas = Canvas2tCorrelation()
        self.__nav = NavigationToolbar(self.__canvas, self)
        
        # Add to layout
        self.__layout.addWidget(self.__nav)
        self.__layout.addWidget(self.__canvas)
        
        self.__optlayout = qtw.QHBoxLayout()
        self.__layout.addLayout(self.__optlayout)
        
        # Data selection  
        self.__grp0 = qtw.QGroupBox("Data selection")
        self.__optlayout.addWidget(self.__grp0)
        
        self.__grp0l= qtw.QFormLayout()
        self.__grp0.setLayout(self.__grp0l)
        
        # Index selection checkboxes
        self.__optindexlayout = qtw.QFormLayout()
        self.__optlayout.addLayout(self.__optindexlayout)
        self.__indexescombo = qtw.QComboBox()
        self.__grp0l.addRow("q-index:", self.__indexescombo)
        
        # Plot selection
        self.__grp1 = qtw.QGroupBox("Plot settings")
        self.__optlayout.addWidget(self.__grp1)
        
        self.__grp1l= qtw.QFormLayout()
        self.__grp1.setLayout(self.__grp1l)
        
        self.__plottype = qtw.QComboBox()
        self.__plottype.insertItem(Canvas2tCorrelation.PLOT_TS_AGE_LOG, 'lag time(log); age(lin)')
        self.__plottype.insertItem(Canvas2tCorrelation.PLOT_TS_AGE_LIN, 'lag time(lin); age(lin)')
        self.__plottype.insertItem(Canvas2tCorrelation.PLOT_XY, 'Frame(lin); Frame(lin)')
        self.__grp1l.addRow("Plot type", self.__plottype)
        
        
        self.__lagsparcification = qtw.QComboBox()
        self.__lagsparcification.addItem('None', Canvas2tCorrelation.NO_SPARCIFICATION)
        self.__lagsparcification.addItem('Lin', Canvas2tCorrelation.LIN_SPARCIFICATION)
        self.__lagsparcification.addItem('Log', Canvas2tCorrelation.LOG_SPARCIFICATION)
        self.__grp1l.addRow("Lag time resamping", self.__lagsparcification)
        
        self.__Nlagresample = qtw.QSpinBox()
        self.__Nlagresample.setMinimum(1)
        self.__Nlagresample.setMaximum(100000)
        self.__Nlagresample.setValue(256)
        self.__grp1l.addRow('Number of lag points', self.__Nlagresample)
        
        self.__agesparcification = qtw.QComboBox()
        self.__agesparcification.addItem('None', Canvas2tCorrelation.NO_SPARCIFICATION)
        self.__agesparcification.addItem('Lin', Canvas2tCorrelation.LIN_SPARCIFICATION)
        self.__agesparcification.addItem('Log', Canvas2tCorrelation.LOG_SPARCIFICATION)
        self.__grp1l.addRow("Age resamping", self.__agesparcification)
        
        self.__Nageresample = qtw.QSpinBox()
        self.__Nageresample.setMinimum(1)
        self.__Nageresample.setMaximum(100000)
        self.__Nageresample.setValue(1000)   
        self.__grp1l.addRow('Number of age points', self.__Nageresample)     
        
        self.__opt2group = qtw.QGroupBox("Colorbar")
        self.__optlayout.addWidget(self.__opt2group)
        
        # self.__opt2group.setCheckable(True)
        # self.__opt2group.setChecked(False)
        self.__opt2layout = qtw.QFormLayout()
        self.__opt2group.setLayout(self.__opt2layout)
        
        self.__mincolorbar = qtw.QDoubleSpinBox()
        self.__opt2layout.addRow("Minimum:", self.__mincolorbar)
        self.__mincolorbar.setMinimum(0)
        self.__mincolorbar.setMaximum(100)
        self.__mincolorbar.setDecimals(4)
        self.__mincolorbar.setSingleStep(.01)
        self.__mincolorbar.setValue(.8)
        
        self.__maxcolorbar = qtw.QDoubleSpinBox()
        self.__opt2layout.addRow("Maximum:", self.__maxcolorbar)
        self.__maxcolorbar.setMinimum(0)
        self.__maxcolorbar.setMaximum(100)
        self.__maxcolorbar.setDecimals(4)
        self.__maxcolorbar.setSingleStep(.01)
        self.__maxcolorbar.setValue(1.5)
        
        self.__colorbarscale = qtw.QComboBox()
        self.__colorbarscale.addItems(['jet', 'ocean', 'gist_earth', 'terrain',
                      'gist_stern', 'gnuplot', 'gnuplot2', 'CMRmap',
                      'cubehelix', 'brg', 'gist_rainbow', 'rainbow', 'jet',
                      'turbo', 'nipy_spectral', 'gist_ncar', 'Greys', 'Purples', 'Blues', 'Greens', 'Oranges', 'Reds',
                      'YlOrBr', 'YlOrRd', 'OrRd', 'PuRd', 'RdPu', 'BuPu',
                      'GnBu', 'PuBu', 'YlGnBu', 'PuBuGn', 'BuGn', 'YlGn', 'gist_yarg', 'gist_gray', 'gray', 'bone',
                      'pink', 'spring', 'summer', 'autumn', 'winter', 'cool',
                      'Wistia', 'hot', 'afmhot', 'gist_heat', 'copper', 'PiYG', 'PRGn', 'BrBG', 'PuOr', 'RdGy', 'RdBu', 'RdYlBu',
                      'RdYlGn', 'Spectral', 'coolwarm', 'bwr', 'seismic', 'twilight', 'twilight_shifted', 'hsv'])
        self.__opt2layout.addRow("Colormap: ", self.__colorbarscale)

        
    def plot(self, files):
        indexes = np.empty((0,),dtype=np.int16)
        cindex = self.__indexescombo.currentIndex()
        
        N = 0
        for f,d in files.items():
            print(f, d)
            with XPCSResultFile(f) as fd:
                indexes = np.append(indexes,np.arange(len(fd.get_q_values(d[0])), dtype=np.int16))
                N += len(files[f])
            
        indexes = np.sort(np.unique(indexes))
        
        print(indexes)
        
        self.__indexescombo.clear()
        self.__indexescombo.addItems(["q-index = %i"%i for i in indexes])
        self.__indexescombo.setCurrentIndex(0 if cindex == -1 else cindex)
                
        cindex = self.__indexescombo.currentIndex()
        if cindex == -1 or cindex >= len(indexes):
            logger.warning("cindex: %i, indexes: %s",cindex, str(indexes))
            cindex=0
            
        if len(indexes) == 0:
            return
        
        self.__canvas.setup(N, indexes[cindex], self.__colorbarscale.currentText(), self.__mincolorbar.value(), self.__maxcolorbar.value())
        
        j = 0
        for f,v in files.items():
            with XPCSResultFile(f) as fd:
                for k in v:
                    self.__canvas.plot(fd,k,j, self.__plottype.currentIndex(),
                                       self.__lagsparcification.currentData(), self.__Nlagresample.value(), 
                                       self.__agesparcification.currentData(), self.__Nageresample.value())
                    j += 1
            
        self.__canvas.draw()
        
        # Remove motion event, this just make the program freeze with that event
        while 'motion_notify_event' in self.__canvas.callbacks.callbacks:
            self.__canvas.callbacks.callbacks.pop('motion_notify_event')
    
    def clear(self):
        self.__canvas.clear()
        
        
class Canvas2tCorrelation(FigureCanvas):
    
    PLOT_TS_AGE_LOG = 0
    PLOT_TS_AGE_LIN = 1
    PLOT_XY         = 2
    
    NO_SPARCIFICATION = None
    LIN_SPARCIFICATION = "lin"
    LOG_SPARCIFICATION = "log"
    
    def __init__(self, parent=None, width=10, height=10, dpi=cmm.default_dpi):
        self.__fig = Figure(figsize=(width, height), dpi=dpi, tight_layout=True)
        super().__init__(self.__fig)
        
        self.setMaximumHeight(100000)
        self.setSizePolicy(qtw.QSizePolicy.Expanding, qtw.QSizePolicy.Expanding)
        
    def setup(self, N, qidx, cmapname='jet', cmin=.5, cmax=2):
        
        logger.debug([N, qidx])
        
        self.__qidx = qidx
        
        nCols = int(np.floor(N**0.5))
        nRows = int(np.ceil(N/nCols))
        
        gs = self.__fig.add_gridspec(nRows, nCols+1,width_ratios=[*[30]*nCols,1])
        
        self.__axes = []
        
        for y in range(nRows*nCols):
            self.__axes += [self.__fig.add_subplot(gs[y//nCols,y%nCols])]
                
        self.__axcb = self.__fig.add_subplot(gs[:,-1])
        
        self.__cmap = plt.get_cmap(cmapname)
        self.__norm = mpl.colors.Normalize(vmin=cmin, vmax=cmax)
        cb = self.__fig.colorbar(mpl.cm.ScalarMappable(norm=self.__norm, cmap=self.__cmap), cax=self.__axcb)        
        cb.set_label("$g_2$")
        
    def plot(self, file, k, j, plot_type = 0, lagsparse=None, lagN=0, agesparse=None, ageN=0):
        
            
        ax = self.__axes[j]
        
        try:
            ttcf, ts, age = file.get_ttcf(k, self.__qidx)
            
            logger.debug("Data read")
        except Exception as e:
            logger.warning(f"Unable to read TTCF\n{e}")
            return
        
        logger.debug(f"{ttcf.shape} {ts.shape} {age.shape}")
        logger.debug(f"{lagN}, {ageN}, {lagsparse}, {agesparse}")
        if lagsparse is not None or agesparse is not None:
            ts, age, ttcf = gt.ttcf_resample_npoints(ttcf, ts, age, lagN, ageN, lagsparse, agesparse)
            logger.debug((ttcf, ts, age))
            logger.debug(f"{ttcf.shape} {ts.shape} {age.shape}")
        
        logger.debug((np.sum(np.isnan(ts)), np.sum(np.isnan(age)),))
        
        mskage = ~np.isnan(age)
        msklag = ~np.isnan(ts)
        logger.debug(f"{ttcf.shape} {mskage.shape} {msklag.shape}")
        
        X,Y = np.meshgrid(ts[msklag], age[mskage])
        
        if plot_type == self.PLOT_XY:
#            dt = age[1]-age[0]
            ax.pcolormesh((.5*X+Y),(Y-.5*X),ttcf[mskage,:][:,msklag], cmap=self.__cmap, norm=self.__norm, rasterized=True)
            ax.pcolormesh((Y-.5*X),(.5*X+Y),ttcf[mskage,:][:,msklag], cmap=self.__cmap, norm=self.__norm, rasterized=True)
            
            ax.set_xlabel('$t_1 (s)$')
            ax.set_ylabel('$t_2 (s)$')
            
            ax.set_aspect('equal', 'box')
            ax.set_xlim([0,np.nanmax(age)])
            ax.set_ylim([0,np.nanmax(age)])
        else:
            ax.pcolormesh(X,Y,ttcf[mskage,:][:,msklag], cmap=self.__cmap, norm=self.__norm, rasterized=True, snap=True, shading='nearest')                
        
            if plot_type == self.PLOT_TS_AGE_LOG:
                ax.set_xscale('log')
                ts = ts[1:]
            else:
                ax.set_aspect('equal', 'box')
                
            ax.set_xlim([np.nanmin(ts),np.nanmax(ts)])
            ax.set_ylim([0,np.nanmax(age)])
            
            ax.set_xlabel(r'Lag time $\tau~(s)$')
            ax.set_ylabel(r'Age $t~(s)$')
        
        qs = file.get_q_values(file.analysis_keys[0])
        
        ax.set_title('%s %s %s q=%.3e'%(file.title, file.filename, k, qs[self.__qidx-1]), fontsize=cmm.default_title_font_size)
        
    def clear(self):
        self.__fig.clf()





# -*- coding: utf-8 -*-

import matplotlib
matplotlib.use('Qt5Agg')

import PyQt5.QtWidgets as qtw

import numpy as np

import logging
logger = logging.getLogger(__name__)

class paramwidget(qtw.QWidget, object):
    def __init__(self):
        super().__init__()
        
        self.__layout = qtw.QGridLayout()
        self.__layout.setColumnStretch(0,1)
        self.__layout.setColumnStretch(1,2)
        self.__layout.setColumnStretch(2,1)
        self.__layout.setColumnStretch(3,2)
        self.__layout.setColumnMinimumWidth(1,100)
        self.__layout.setColumnMinimumWidth(3,100)
        self.setLayout(self.__layout)
        
        self.__value = qtw.QLineEdit()
        self.__mvalue = qtw.QLineEdit()        
        self.__Mvalue = qtw.QLineEdit()
        
        self.__isFixedValue = qtw.QCheckBox()
        self.__isFixedValue.setChecked(False)
        
        self.__isMultipleValue = qtw.QCheckBox()
        self.__isMultipleValue.setChecked(True)
        
        self.__layout.addWidget(qtw.QLabel("Initial: "),0,0)
        self.__layout.addWidget(self.__value,0,1)
        
        self.__layout.addWidget(qtw.QLabel("Fixed: "),0,2)
        self.__layout.addWidget(self.__isFixedValue,0,3)
        
        self.__layout.addWidget(qtw.QLabel("Minimum: "),1,0)
        self.__layout.addWidget(self.__mvalue,1,1)
        
        self.__layout.addWidget(qtw.QLabel("Maximum: "),1,2)
        self.__layout.addWidget(self.__Mvalue,1,3)
        
        self.__layout.addWidget(qtw.QLabel("One value per curve: "),2,0,1,3)
        self.__layout.addWidget(self.__isMultipleValue,2,3)
        
        self.setValue(0.)
        self.setMinval(0.)
        self.setMaxval(+np.inf)
    
    def value(self):
        return float(self.__value.text())
    
    def setValue(self, val : float):
        self.__value.setText('%.5e'%val)
        
    def minval(self):
        return float(self.__mvalue.text())
    
    def setMinval(self, mval : float):
        self.__mvalue.setText('%.5e'%mval)
        
    def maxval(self):
        return float(self.__Mvalue.text())
    
    def setMaxval(self, mval : float):
        self.__Mvalue.setText('%.5e'%mval)
    
    def isFixed(self):
        return self.__isFixedValue.isChecked()
    
    def setIsFixed(self, fix : bool):
        self.__isFixedValue.setChecked(fix)
        
    def isMultipleValue(self):
        return self.__isMultipleValue.isChecked()
    
    def setIsMultipleValue(self, mul : bool):
        self.__isMultipleValue.setChecked(mul)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Oct  7 18:32:25 2021

@author: opid02
"""

import matplotlib.pyplot as plt
import matplotlib as mpl
import sys
import logging
import numpy as np
from .result_file import XPCSResultFile
import scipy.optimize as opt

#plt.rcParams.update({
#    "text.usetex": False,
#    "font.family": "sans-serif",
#    "font.sans-serif": ["Helvetica"]})

logger = logging.getLogger(__name__)

def qindex_colors(qidx, colormap='default'):
    
    if colormap == 'default':
        cmap = mpl.cm.terrain
        maxval = np.max(qidx)/.76
        norm = mpl.colors.Normalize(vmin=np.min(qidx), vmax=maxval)
        
        cmap, norm = truncate_colormap(cmap, norm)
        return (cmap, norm, maxval)
    else:
        cmap = plt.get_cmap(colormap)
        maxval = np.max(qidx)
        norm = mpl.colors.Normalize(vmin=np.min(qidx), vmax=maxval)
        return (cmap, norm, maxval)
    
def symbol(i):
    sym = ['o', 's', '^', 'v', '>', '<', 'd']
    
    sid = i%len(sym)
    return sym[sid]
        
def truncate_colormap(cmap, norm, minval=0.0, maxval=0.76, n=100):
    new_cmap = mpl.colors.LinearSegmentedColormap.from_list(
        'trunc({n},{a:.2f},{b:.2f})'.format(n=cmap.name, a=minval, b=maxval),
        cmap(np.linspace(minval, maxval, n)))
    norm2 = mpl.colors.Normalize(vmin=norm.vmin, vmax=norm.vmax*maxval)
    return new_cmap, norm2

def plot_SAXS_with_qranges(file):
    ax = plt.subplot(111)
    ax.plot(file.saxs_curve[:,0], file.saxs_curve[:,1])
    
    (cmapi, normi, mv) = qindex_colors(file.q_ranges[:,0])
    
    for i,qi in enumerate(file.q_ranges[:,0]):
        ax.fill_between(file.saxs_curve[:,0], file.saxs_curve[:,1], where=np.logical_and(file.saxs_curve[:,0] >= file.q_ranges[i,1], file.saxs_curve[:,0] <= file.q_ranges[i,2]), color=cmapi(normi(qi)))
    
    ax.set_xscale('log')
    ax.set_yscale('log')
    ax.set_xlabel('$q (nm^-1)$')
    ax.set_ylabel('$I(q)$')
    
def plot_analysis_qranges(file, eta=0.89e-23, T=298, exp=1.):
    
    fig = plt.gcf()
    gs = fig.add_gridspec(3,len(file.analysis_keys)*2+1,width_ratios=[*[30/len(file.analysis_keys)]*(2*len(file.analysis_keys)), 1])
    
    ax = None
    
    # SAXS pattern colormap
    patt = file.saxs_pattern
    
    cmap = mpl.cm.jet
    norm = mpl.colors.LogNorm()
    
    # qmask colorbar
    q_ranges = file.q_ranges
    qs = (q_ranges[:,1]+q_ranges[:,2])/2.
    (cmapi, normi, mv) = qindex_colors(q_ranges[:,0])
    
    # Hydraulic radius
    kB = 1.38e-23
    def Rh(k):
        return (kB*T)/(6.*np.pi*eta*k)
    
    # fit functions
    def model(t,beta,D, sf):
        return 1. + beta*np.exp(-2.*D*t**sf)
    
    def propmodel(q,k):
        return k*q
    
    def sqmodel(q,k):
        return k*q**2
    
    bnd_st = (1.,0, +np.inf)
    
    if exp is not None:
        bnd_st = [exp,exp, exp+0.00001]
    
    for i,k in enumerate(file.analysis_keys):
        if ax is None:
            ax1 = fig.add_subplot(gs[0,i*2:i*2+2])
            ax = ax1
            #cb1 = mpl.colorbar.ColorbarBase(ax, cmap=cmap, norm=norm)
        else:
            ax1 = fig.add_subplot(gs[0,i*2:i*2+2], sharex=ax, sharey=ax)            
        
        ax1.imshow(patt, cmap=cmap, norm=norm)
        qmask = np.array(file.get_qmask(k), np.float32)
        qmask[qmask == 0] = np.nan
        qmask[file.mask == 0] = mv
        ax1.imshow(qmask, cmap=cmapi, norm=normi, alpha=.6)
        
        (ts, cf, std) = file.get_correlations(k)
        kk = np.zeros((cf.shape[1],3))
        
        ax2 = fig.add_subplot(gs[1,i*2:i*2+2])
        for ii,qi in enumerate(file.get_q_index(k)):
            ax2.plot(ts, cf[:,ii], ".", color=cmapi(normi(qi)))
            
            msk = np.isnan(cf[:,ii]) == False
            
            k0 = opt.curve_fit(model, ts[msk].ravel(),cf[msk,ii], [0.3,1e2,bnd_st[0]], bounds=([0,0,bnd_st[1]],[1,+np.inf,bnd_st[2]]))
            ax2.plot(ts, model(ts,*k0[0]), ':', color=cmapi(normi(qi)))
            kk[ii,:] = k0[0]
            
        ax2.set_xscale('log')
        
        ax3 = fig.add_subplot(gs[2,i*2])
        ax4 = fig.add_subplot(gs[2,i*2+1])
        
        qq = np.array([qs[i-1] for i in file.get_q_index(k)])
        ax3.plot(qq, kk[:,0], 'o')
        ax4.plot(qq, kk[:,1], 'o')
        
        klog = np.polyfit(np.log(qq),np.log(kk[:,1]),1)
        klin = opt.curve_fit(propmodel, qq, kk[:,1], np.mean(kk[:,1]/qq))
        ksq  = opt.curve_fit(sqmodel, qq, kk[:,1], np.mean(kk[:,1]/qq**2))
        
        gamma_log = np.exp(np.polyval(klog,np.log(qq)))
        gamma_lin = propmodel(qq, klin[0][0])
        gamma_sq  = sqmodel(qq, ksq[0][0])
        
        r2 = lambda g: 1.-np.sum((kk[:,1]-g)**2)/np.sum((kk[:,1]-np.mean(kk[:,1]))**2)
        
        ax4.plot(qq,gamma_log, label='$%.03eq^{%.02f}, r^2=%.3f$'%(np.exp(klog[1]),klog[0], r2(gamma_log)))
        ax4.plot(qq,gamma_lin, label='$%.03eq, r^2=%.3f$'%(klin[0][0], r2(gamma_lin)))
        ax4.plot(qq,gamma_sq , label='$%.03eq^2, Rh=%.1f~nm$, r^2=%.3f'%(ksq[0][0],1e9*Rh(ksq[0][0]), r2(gamma_sq)))
        ax4.legend()
        
        ax4.set_xscale('log'); ax4.set_yscale('log')
        
        ax2.set_xlabel('$ts (s)$'); ax2.set_ylabel('$g_2$')
        ax3.set_xlabel('$q (nm^{-1})$'); ax3.set_ylabel('$\\beta$')
        ax4.set_xlabel('$q (nm^{-1})$'); ax4.set_ylabel('$\\Gamma(q)$')
        
    ax1 = fig.add_subplot(gs[0,-1])
    ax2 = fig.add_subplot(gs[1,-1])
    
    fig.colorbar(mpl.cm.ScalarMappable(norm=norm, cmap=cmap), cax=ax1, label="I(q)")
    fig.colorbar(mpl.cm.ScalarMappable(norm=normi, cmap=cmapi), cax=ax2, label="q", format=mpl.ticker.FuncFormatter(lambda x,p: "%.3e"%(qs[np.where(x == q_ranges[:,0])])), ticks=q_ranges[::((q_ranges.shape[0]//16)+1),0])
    

def plot_ttcf(file, key, idx):
    ttcf, ts, age = file.get_ttcf(key, idx)
    fig = plt.gcf()
    ax = plt.subplot(111)
    
    cmap = mpl.cm.jet
    norm = mpl.colors.Normalize(vmin=0.5, vmax=2)
    
    X,Y = np.meshgrid(ts[1:], age)
    ax.pcolormesh(X,Y,ttcf[:,1:], cmap=cmap, norm=norm)
    
    ax.set_xscale('log')
    ax.set_xlabel('ts (s)')
    ax.set_ylabel('Age (s)')
    ax.set_title('%s / %i q=%.3e'%(key, idx, file.qs[file.get_q_index(key)[idx]-1]))
    
    fig.colorbar(mpl.cm.ScalarMappable(norm=norm, cmap=cmap), label="ttcf(age,lag)")
    

def main():
    logger.setLevel(logging.DEBUG)
    file = XPCSResultFile(sys.argv[1])
    
    plt.figure(0)
    plot_SAXS_with_qranges(file)
    
    plt.figure(1)
    plot_analysis_qranges(file)
    
    for i,k in enumerate(file.analysis_keys):
        plt.figure(2)
        plot_ttcf(file, k, 3)
        break
    
    plt.show()
    
    
def _matrixAvg(x, nx_intvs, M, axis):
    """
    Resample a matrix along specific axis
    
    :param x: Axis coordinates. Have to be strictly increasing.
    :param nx_intvs: Intervals at which the matrix will be sliced
    :param M: Matrix to be sliced
    :param axis: Axis number
    
    :return (nx, nM): New axis and new matrix
    """
    
    x = x.ravel()
    nx_intvs = nx_intvs.ravel()
    if x.shape[0] != M.shape[axis]:
        raise ValueError("Inconsistant axis coordinates with matrix.")
        
    if not np.all(np.diff(x) > 0):
        raise ValueError("x Have to be strictly increasing.")
        
    # Compute slicing indexes
    # idx = np.arange(x.shape[0])
    # sidx = []
    # for v in nx_intvs:
    #     if v < x[-1]:
    #         sidx += [idx[x > v][0],]
    #     else:
    #         sidx += [x.shape[0]-1,]
            
    # binc = np.diff(np.r_[sidx,x.shape[0]])
    # nx = np.add.reduceat(x, sidx)/binc
    # nM = np.add.reduceat(M, sidx, axis)/binc
    
    logger.debug(M)
    
    N = nx_intvs.shape[0]-1
    nx = np.zeros((N), dtype=np.float32)*np.nan
    MS = [*M.shape]
    MS[axis] = N
    nM = np.zeros(MS, dtype=np.float32)*np.nan
    # logger.debug(M.shape)
    
    idx1 = [slice(None),]*len(MS)
    idx2 = [slice(None),]*len(MS)
    MM = []
    
    for j in range(N):
        msk = np.logical_and(x >= nx_intvs[j], x < nx_intvs[j+1])
        # logger.debug(np.any(msk))
        if np.any(msk):
            nx[j] = np.nanmean(x[msk])
            idx1[axis] = j
            idx2[axis] = msk
            
            avg = np.nanmean(M.__getitem__(idx2), axis=axis)
            
            # logger.debug(avg.shape)
            
            nM.__setitem__(idx1, avg)
            
    # logger.debug((nx, nM))
    
    idxs = np.where(np.isnan(nx))
    nM = np.delete(nM, idxs, axis)
    nx = np.delete(nx, idxs, 0)
    
    return nx, nM
    
def ttcf_resample(ttcf, lag, age, lag_intervals=None, age_intervals=None):
    """
    Resample the two time correlation function, by averaging
    
    :param ttcf:            (LxA) The two time correlation function to be resampled
    :param lag:             (1XL) TTCF lag time axis
    :param age:             (1XA) TTCF age axis
    :param lag_intervals:   (1xnL+1) Bounds of new lag time axis. If none, do not rescale this axis
    :param age_intervals:   (1xnA+1) Bounds of new age axis. If none, do not rescale this axis
    
    :return (nlag, nage, nttcf): 
    """
    
    reshape = False
    if len(ttcf.shape) == 2:
        ttcf.shape = (1,*ttcf.shape)
        reshape = True
        
    lag = lag.ravel()
    age = age.ravel()
    
    if lag_intervals is not None:
        lag, ttcf = _matrixAvg(lag, lag_intervals, ttcf, 2)
        
    logger.debug("lag time done")
        
    if age_intervals is not None:
        age, ttcf = _matrixAvg(age, age_intervals, ttcf, 1)
        
    logger.debug("age done")
        
    if reshape:
        ttcf.shape = ttcf.shape[1:]
    
    return lag, age, ttcf


def ttcf_resample_npoints(ttcf, lag, age, nlag=0, nage=0, lagmode=None, agemode=None):
    """
    Resample the TTCF by specifying the number of points
    
    :param ttcf: The ttcf to be resampled
    :param lag: Lag time axis
    :param age: Age axis
    :param nlag: Number of points along the lag time axis
    :param nage: Number of points along the age axis
    :param lagmode: Mode for lag time. None for no reduction, 'lin' or 'log'.
    :param agemode: Mode for age. None for no reduction, 'lin' or 'log'.
    """
    
    lag_intervals = None
    age_intervals = None
    
    if lagmode is not None:
        if lagmode == 'lin':
            lag_intervals = np.linspace(0, np.max(lag), nlag+1)
        elif lagmode == 'log':
            msk = lag > 0
            lag_intervals = 10**np.linspace(np.log10(np.min(lag[msk])), np.log10(np.max(lag)), nlag+1)
        else:
            raise ValueError(f"Lagmode should be None, lin or log, not {lagmode}")
    
    if agemode is not None:
        if agemode == 'lin':
            age_intervals = np.linspace(0, np.max(age), nage+1)
        elif agemode == 'log':
            msk = age > 0
            age_intervals = 10**np.linspace(np.log10(np.min(age[msk])), np.log10(np.max(age)), nage+1)
        else:
            raise ValueError(f"Agemode should be None, lin or log, not {agemode}")
            
    return ttcf_resample(ttcf, lag, age, lag_intervals, age_intervals)
        
        
        
        
        
        
        
        
        
        
        



# -*- coding: utf-8 -*-

import PyQt5.QtWidgets as qtw

import numpy as np

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas, NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

import xpcsutilities.tools.graphs as gt
import xpcsutilities.tools.common as cmm
import xpcsutilities.tools.contin as contin

class ContinResult(qtw.QWidget):
    """
    Display the result of CONTIN analysis
    """
    
    def __init__(self):
        """
        Initialize the view with graph
        """
        
        super().__init__()
        
        self.__layout = qtw.QGridLayout()
        self.setLayout(self.__layout)
        
        self.__Gcanvas = Contin2DCanvas()
        self.__navG = NavigationToolbar(self.__Gcanvas, self)
        
        self.__layout.addWidget(self.__navG, 0,0)
        self.__layout.addWidget(self.__Gcanvas, 1,0)
        
        self.__g2canvas = Contin1DCanvas()        
        self.__navg2 = NavigationToolbar(self.__g2canvas, self)
        self.__layout.addWidget(self.__navg2, 0,1)        
        self.__layout.addWidget(self.__g2canvas, 1,1)
        
        self.__residuecanvas = Contin1DCanvas()  
        self.__navres = NavigationToolbar(self.__residuecanvas, self)
        self.__layout.addWidget(self.__navres, 2,1)           
        self.__layout.addWidget(self.__residuecanvas, 3,1)
        
        
    def plot(self, q, tau, cf, s, beta, G):
        """
        Display!
        """
        
        self.__Gcanvas.plot(q, s, G*s[:,None])
        
        cfmod = np.zeros_like(cf)
        for i, qv in enumerate(q):
            cfmod[:,i] = contin.g2LT(s, G[:,i], tau, beta[i])
            
        self.__g2canvas.plot(tau, cf-1, q, 'o')
        self.__g2canvas.plot(tau, cfmod, q, '-')
        
        self.__residuecanvas.plot(tau, cfmod-(cf-1), q)
        
        

class Contin1DCanvas(FigureCanvas):
    """
    Display 1d curves with curves colorized with q
    """
    
    def __init__(self, parent=None, width=5, height=5, dpi=cmm.default_dpi):
        self.__fig = Figure(figsize=(width, height), dpi=dpi, tight_layout=True)
        super().__init__(self.__fig)
        
        gs = self.__fig.add_gridspec(1, 2, width_ratios=[35, 1])
        self.__ax  = self.__fig.add_subplot(gs[0,0]) 
        self.__cax = self.__fig.add_subplot(gs[0,1]) 
        
    def plot(self, x, y, c, *opts, **kopts):
        """
        Plot y(x, c)
        """
        
        cmap = mpl.cm.jet
        norm = mpl.colors.LogNorm(vmin=np.min(c), vmax=np.max(c))
        
        for i, cv in enumerate(c):
            self.__ax.plot(x, y[:,i], *opts, color=cmap(norm(cv)), **kopts)
            
            
        self.__ax.set_xscale('log')
        self.__ax.set_xlabel('$\tau (s)$')
        self.__ax.set_ylabel('$g_2-1$')
        self.__fig.colorbar(mpl.cm.ScalarMappable(norm=norm, cmap=cmap), cax=self.__cax, label='$q (nm^{-1})$')
        
        
class Contin2DCanvas(FigureCanvas):
    """
    Display 2d contin map
    """
    
    def __init__(self, parent=None, width=5, height=5, dpi=cmm.default_dpi):
        self.__fig = Figure(figsize=(width, height), dpi=dpi, tight_layout=True)
        super().__init__(self.__fig)
        
        gs = self.__fig.add_gridspec(1, 2, width_ratios=[35, 1])
        self.__ax  = self.__fig.add_subplot(gs[0,0]) 
        self.__cax = self.__fig.add_subplot(gs[0,1]) 
        
    def plot(self, x, y, z, **kopts):
        """
        Plot z(x, y)
        """
        
        cmap = mpl.cm.jet
        norm = mpl.colors.Normalize(vmin=np.min(z), vmax=np.max(z))
        
        self.__ax.pcolormesh(x, y, z, cmap=cmap, norm=norm, **kopts)
        self.__ax.set_xscale('log')
        self.__ax.set_yscale('log')
        
        self.__ax.set_xlabel('$q (nm^{-1})$')
        self.__ax.set_ylabel('$s (s)$')
            
        self.__fig.colorbar(mpl.cm.ScalarMappable(norm=norm, cmap=cmap), cax=self.__cax, label='$G*s$')





# -*- coding: utf-8 -*-


import matplotlib
matplotlib.use('Qt5Agg')

import PyQt5.QtWidgets as qtw

import numpy as np

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas, NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
from matplotlib.markers import MarkerStyle
import matplotlib.cm as cm
import xpcsutilities.tools.graphs as gt

import logging
logger = logging.getLogger(__name__)
        
class ResultFitGraph(qtw.QWidget):
    def __init__(self, results, cmap=None, norm=None):
        super().__init__()
        
        self.__results = results
        
        #Layout
        self.__layout = qtw.QVBoxLayout()
        self.setLayout(self.__layout)
        
        #Toolbar, figures
        self.__canvas = ResultFitCanvas(cmap, norm)
        self.__nav = NavigationToolbar(self.__canvas, self)
        
        # Add to layout
        self.__layout.addWidget(self.__nav)
        self.__layout.addWidget(self.__canvas)  
        
        # Display options toolbar
        self.__toolbarlayout = qtw.QHBoxLayout()
        self.__layout.addLayout(self.__toolbarlayout)
        
        self.__opt1layout = qtw.QFormLayout()
        self.__toolbarlayout.addLayout(self.__opt1layout)
        
        self.__displayG2_1 = qtw.QCheckBox()
        self.__displayG2_1.setChecked(True)
        self.__opt1layout.addRow("Display rescaled $(g_2-1)/\\beta$", self.__displayG2_1)
        
        self.__displaySep = qtw.QCheckBox()
        self.__displaySep.setChecked(False)
        self.__opt1layout.addRow("Display each functions", self.__displaySep)
        
        self.__btnredraw = qtw.QPushButton("Redraw")
        self.__btnredraw.setMinimumHeight(100)
        self.__toolbarlayout.addWidget(self.__btnredraw)
        self.__btnredraw.clicked.connect(self.plot)
        
        self.plot()
        
    def plot(self):
        self.__canvas.clear()
        self.__canvas.plot(self.__results, self.__displayG2_1.isChecked(), self.__displaySep.isChecked())
        self.__canvas.draw()

class ResultFitCanvas(FigureCanvas):
    def __init__(self, cmap=None, norm=None, parent=None, width=10, height=10, dpi=150):
        self.__fig = Figure(figsize=(width, height), dpi=dpi, tight_layout=True)
        self.__axes = self.__fig.add_subplot(111)
        self.__cmap = cmap
        self.__norm = norm
        self.__cb   = None
        super().__init__(self.__fig)
        
    def plot(self, results, rescale=False, sep=False):
        
        if len(results) > 0 and not 'beta' in results[0]['fitres'] and rescale:
            logger.critical("No contrast (beta) found.")
            qtw.QMessageBox.critical(self, "No contrast", "Can't rescale according to contrast: parameter beta not found")
            rescale = False
            
        for r in results:
            
            cf = r['cf']
            lag = r['lag']
            
            if rescale:
                cf = (cf-1.)/r['fitres']['beta']
                
            plargs = dict()
            
            if self.__cmap is not None and self.__norm is not None:
                plargs['color'] = self.__cmap(self.__norm(r['color']))
                
            self.__axes.plot(lag, cf, 'o', **plargs)
                
            if 'fitfun' in r:
                
                lxm = np.floor(np.log10(min(lag)))
                lxM = np.ceil(np.log10(max(lag)))
                    
                x = np.logspace(lxm, lxM, int(50*(lxM-lxm)))
                
                prms = []
                qs = np.array([r['q'],])
                for i,k in enumerate(r['fitkeys']):
                    prms += [ r['fitres'][k], ]
                    
                funv, funsep, N = r['fitfun'](x,qs,*prms, sepf=True, scale=False)
                
                if 'beta' in r['fitkeys']:
                    if not rescale:
                        funv += 1
                        funsep += 1
                    else:
                        funv = (funv)/r['fitres']['beta']
                        funsep = (funsep)/r['fitres']['beta']                        
                else:
                    if not rescale:
                        funv = (funv)*r['fitres']['beta']+1.
                        funsep = (funsep)*r['fitres']['beta']+1.
                    
                self.__axes.plot(x, funv, **plargs)
                    
                if N>1 and sep :
                    self.__axes.plot(x, funsep, '--', **plargs)
        
        self.__axes.set_xscale('log')
        self.__axes.set_yscale('linear')
        self.__axes.set_xlabel('$lag time (s)$')
        self.__axes.set_ylabel('$(g_2-1)/\\beta$' if rescale else '$g_2$')
        
        if self.__cb is None:
            (cmap, norm) = gt.truncate_colormap(self.__cmap, self.__norm)
            self.__cb = self.__fig.colorbar(cm.ScalarMappable(norm=norm, cmap=cmap))
        
        #self.__axes.legend(fontsize=7)
        
    def clear(self):
        self.__axes.cla()
        

class ResultParamsFitGraph(qtw.QWidget):
    def __init__(self, results):
        super().__init__()
        
        self.__results = results
        
        #Layout
        self.__layout = qtw.QVBoxLayout()
        self.setLayout(self.__layout)
        
        #Toolbar, figures
        self.__canvas = ResultParamsFitCanvas(self.__results['params'].keys())
        self.__nav = NavigationToolbar(self.__canvas, self)
        
        # Add to layout
        self.__layout.addWidget(self.__nav)
        self.__layout.addWidget(self.__canvas)  
        
        # Display options toolbar
        self.__toolbarlayout = qtw.QHBoxLayout()
        self.__layout.addLayout(self.__toolbarlayout)
        
        # OPT1: log axes
        self.__opt1layout = qtw.QFormLayout()
        self.__toolbarlayout.addLayout(self.__opt1layout)
        
        self.__opt_logx = qtw.QCheckBox()
        self.__opt1layout.addRow('Log x', self.__opt_logx)
        
        self.__opt_logy = dict()
        for k in self.__results['params']:
            cb = qtw.QCheckBox()
            self.__opt1layout.addRow('Log %s'%k, cb)
            self.__opt_logy[k] = cb
            
        # opt2: color, marker, xaxis selection
        self.__opt2layout = qtw.QFormLayout()
        self.__toolbarlayout.addLayout(self.__opt2layout)
        
        self.__opt_x_axis = qtw.QComboBox()
        self.__opt_x_axis.addItem('q', ResultParamsFitCanvas.OPT_Q)
        self.__opt_x_axis.addItem('time', ResultParamsFitCanvas.OPT_TIME)
        self.__opt2layout.addRow("x axis", self.__opt_x_axis)
        
        self.__opt_c_axis = qtw.QComboBox()
        self.__opt_c_axis.addItem('None', ResultParamsFitCanvas.OPT_NONE)
        self.__opt_c_axis.addItem('q', ResultParamsFitCanvas.OPT_Q)
        self.__opt_c_axis.addItem('time', ResultParamsFitCanvas.OPT_TIME)
        self.__opt_c_axis.addItem('direction', ResultParamsFitCanvas.OPT_DIR)
        self.__opt_c_axis.setCurrentIndex(2)
        self.__opt2layout.addRow("color axis", self.__opt_c_axis)
        
        self.__opt_m_axis = qtw.QComboBox()
        self.__opt_m_axis.addItem('Direction + file', ResultParamsFitCanvas.OPT_DIRFILE)
        self.__opt_m_axis.addItem('Direction', ResultParamsFitCanvas.OPT_DIR)
        self.__opt_m_axis.addItem('File', ResultParamsFitCanvas.OPT_FILE)
        self.__opt2layout.addRow("Marker", self.__opt_m_axis)
            
        # Redraw button
        self.__btnredraw = qtw.QPushButton("Redraw")
        self.__btnredraw.setMinimumHeight(100)
        self.__toolbarlayout.addWidget(self.__btnredraw)
        self.__btnredraw.clicked.connect(self.plot)
        
        
        files = []
        dirs = []
        mids = []
        
        self.__results['fid'] = []
        self.__results['did'] = []
        self.__results['mid'] = []
        
        for i in range(len(self.__results['q'])):
            if self.__results['file'][i] not in files:
                files += [self.__results['file'][i], ]
                
            if self.__results['dir'][i] not in files:
                dirs += [self.__results['dir'][i], ]
                
            fid = files.index(self.__results['file'][i])
            did = dirs.index(self.__results['dir'][i])
            
            mark = '%i_%i'%(fid, did)
            
            if mark not in mids:
                mids += [mark, ]
                
            mid = mids.index(mark)
            
            self.__results['fid'] += [fid, ]
            self.__results['did'] += [did, ]
            self.__results['mid'] += [mid, ]
            
        self.__results['q'] = np.array(self.__results['q'])
        self.__results['tsr'] = np.array(self.__results['tsr'])
        self.__results['fid'] = np.array(self.__results['fid'])
        self.__results['did'] = np.array(self.__results['did'])
        self.__results['mid'] = np.array(self.__results['mid'])
        
        for k in self.__results['params']:
            self.__results['params'][k] = np.array(self.__results['params'][k])
        
        self.plot()
        
    def plot(self):        
        self.__canvas.clear()
        self.__canvas.plot(self.__results,
                           logx=self.__opt_logx.isChecked(),
                           logy={ k:cb.isChecked() for k,cb in self.__opt_logy.items() },
                           xaxis = self.__opt_x_axis.currentData(),
                           caxis = self.__opt_c_axis.currentData(),
                           mark = self.__opt_m_axis.currentData()
                           )
        self.__canvas.draw()

class ResultParamsFitCanvas(FigureCanvas):
    
    OPT_NONE    = 0
    OPT_Q       = 1
    OPT_TIME    = 2
    OPT_DIR     = 3
    OPT_FILE    = 4
    OPT_DIRFILE = 5
    
    def __init__(self, params, parent=None, width=10, height=10, dpi=150):
        self.__params = params
        self.__fig = Figure(figsize=(width, height), dpi=dpi, tight_layout=True)
        
        N = len(params)
        nc = int(np.ceil(N**.5))
        nr = int(nc - np.floor((nc**2-N)/nc))
        
        ax = self.__fig.subplots(nr,nc,sharex='col', squeeze=False)
        self.__axes = [ a for sub in ax for a in sub ]
        
        super().__init__(self.__fig)
        
    def plot(self, results, logx, logy, xaxis, caxis, mark):
        
        if xaxis == self.OPT_Q:
            x = results['q']
        elif xaxis == self.OPT_TIME:
            x = results['tsr']
        else:
            raise ValueError('Unsuported x axis, %s'%xaxis)
            
        if caxis == self.OPT_Q:
            c = results['q']
        elif caxis == self.OPT_TIME:
            c = results['tsr']
        elif caxis == self.OPT_DIR:
            c = results['did']
        elif caxis == self.OPT_NONE:
            c = None
            cmap, norm = None, None
        else:
            raise ValueError('Unsuported c axis %s'%caxis)
            
        if c is not None:
            cmap, norm, M = gt.qindex_colors(c)
            cmap, norm = gt.truncate_colormap(cmap, norm)
            
        if mark == self.OPT_DIR:
            m = results['did']
        elif mark == self.OPT_FILE:
            m = results['fid']
        elif mark == self.OPT_DIRFILE:
            m = results['mid']
        else:
            raise ValueError('Unsuported m axis %s'%mark)
        
        markers = ('o','s','v','P','^','*','<','D','>','X','p')
        
        for j,M in enumerate(np.unique(m)):
            msk = m == M
            marker_id = np.mod(j, len(markers))
            
            for i,k in enumerate(self.__params):
                self.__axes[i].scatter(x[msk],
                           results['params'][k][msk],
                           c=c[msk] if c is not None else None,
                           cmap=cmap, norm=norm,
                           marker=MarkerStyle(markers[marker_id]))

        for i,k in enumerate(self.__params):
            if logx:
                self.__axes[i].set_xscale('log')
            
            if logy[k]:
                self.__axes[i].set_yscale('log')
            
            self.__axes[i].set_title(k)
            self.__axes[i].grid()
        
    def clear(self):
        for a in self.__axes:
            a.cla()
        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Oct 18 13:29:26 2021

@author: opid02
"""

import numpy as np
import logging
import PyQt5.QtGui as qtg
import PyQt5.QtWidgets as qtw
import PyQt5.QtCore as qtc
import os
from watchdog.observers.polling import PollingObserver
from watchdog.events import PatternMatchingEventHandler
from xpcsutilities.tools.result_file import XPCSResultFile, NotXPCSFileException

logger = logging.getLogger(__name__)


class FileDatabase(object):
    """
    File database
    """
    __db = dict()
    __rdb = dict()
    
    class FileDescriptor(object):
        """
        Describe a file
        """
        
        def __init__(self, filename):
            """
            Read the filename and fill the cache fields
            """
            
            filename = os.path.abspath(filename)
            
            with XPCSResultFile(filename) as fd:
                self.__filename = filename
                self.__keys = fd.analysis_keys
                self.__title = fd.title
                
        @property
        def filename(self):
            return self.__filename
        
        @property
        def keys(self):
            return self.__keys
        
        @property
        def title(self):
            return self.__title
    
    @classmethod
    def getDescriptorFromFilename(cls, filename):
        """
        Return a file descriptor based on filename, creating a new one if needed
        """
        
        if filename not in cls.__db:
            cls.__db[filename] = cls.FileDescriptor(filename)
                
        return cls.__db[filename]
        
    
    @classmethod
    def getItemsFromFilename(cls, filename):
        """
        Create items for filename entries if needed
        """
        
        fd = cls.getDescriptorFromFilename(filename)
        
        if fd is None:
            return None
            
        r = dict()
        
        for k in fd.keys:
            uid = f"{fd.filename}::{k}"
            
            if not uid in cls.__rdb:                
                cls.__rdb[uid] = {'fd': fd, 'key': k}
                
            r[uid] = cls.__rdb[uid]
            
        return r
            
    @classmethod
    def getDescriptorFromKey(cls, uid):
        """
        Return the entry from the item key
        """
        
        if uid in cls.__rdb:
            return cls.__rdb[uid]
        
        raise ValueError(f"Item {uid} not found in database")


class FileDock(qtw.QDockWidget):
    # Signals
    onPlotRequest = qtc.pyqtSignal(dict)
    
    def __init__(self):
        logger.debug("FileDock creation")
        super().__init__()
        self.__currentDir = os.getcwd()
        
        w = qtw.QWidget()
        self.__layout = qtw.QVBoxLayout()
        w.setLayout(self.__layout)
        self.setWidget(w)
        
        # Change directory button
        self.__btnSetFolder = qtw.QPushButton("Set working folder")
        self.__btnSetFolder.clicked.connect(self.setWorkingFolder)
        
        self.__layout.addWidget(self.__btnSetFolder)
        
        # File filter
        self.__filefilter = qtw.QLineEdit("*.h5")
        self.__filefilter.editingFinished.connect(lambda *args: self.setWorkingFolder(self.__currentDir))
        self.__layout.addWidget(self.__filefilter)
        
        # File list
#        self.__filelist = qtw.QListWidget()
#        self.__filelist.setSelectionMode(qtw.QListWidget.ExtendedSelection)
#        self.__layout.addWidget(self.__filelist)
        self.__filelist = qtw.QTreeView()
        
        self.__filemodel = qtg.QStandardItemModel()
        self.__filemodel.setColumnCount(3)
        self.__filemodel.setHeaderData(0, qtc.Qt.Horizontal, 'File')
        self.__filemodel.setHeaderData(1, qtc.Qt.Horizontal, 'Entry')
        self.__filemodel.setHeaderData(2, qtc.Qt.Horizontal, 'Title')
        # self.__filemodel.setHorizontalHeaderLabels(['File', 'Entry'])
        
        self.__filelist.setModel(self.__filemodel)
        self.__filelist.setSelectionMode(qtw.QListWidget.ExtendedSelection | qtw.QListWidget.SelectRows)
        self.__layout.addWidget(self.__filelist)
        
        # ProgressBar
        self.__prgBar = qtw.QProgressBar()
        self.__prgBar.setMinimum(0)
        self.__layout.addWidget(self.__prgBar)
        
        # Update list
        self.__upbtn = qtw.QPushButton("Update List")
        self.__upbtn.setMinimumHeight(50)
        self.__layout.addWidget(self.__upbtn)
        self.__upbtn.clicked.connect(self.updateFiles)
        
        # Plot button
        self.__pltbtn = qtw.QPushButton("Plot")
        self.__pltbtn.setMinimumHeight(100)
        self.__layout.addWidget(self.__pltbtn)
        self.__pltbtn.clicked.connect(self.onPlotButtonPressed)
        
        # File observer
        self.__observer = PollingObserver()
        self.__observer.start()
        
        # Load from current directory
        self.__lastDir = None
        self.__currfiles = dict()
        
        self.setMinimumWidth(400)
        
        
        # self.setWorkingFolder(os.getcwd())
        qtc.QTimer.singleShot(1000, lambda: self.setWorkingFolder(os.getcwd()))
        
        
    def __del__(self):
        self.__observer.stop()
        self.__observer.join()
    
    @property
    def fileFilters(self):
        t = self.__filefilter.text()
        
        if t.strip() == '':
            return ['*',]
        else:
            return [ f.strip() for f in self.__filefilter.text().split(',')]
        
    def setWorkingFolder(self, wd=None):
        
        if not wd:
            nd = qtw.QFileDialog.getExistingDirectory(self, "Change working directory", self.__currentDir, qtw.QFileDialog.ShowDirsOnly)
            if os.path.isdir(nd):
                self.__currentDir = nd
            else:
                logger.warning('Non existant directory: %s', nd)
                return
        else:
            self.__currentDir = wd
            
        os.chdir(self.__currentDir)
            
        self.updateFiles()
        
        # Watch for modifications in the folder
#        ignore_patterns = None
#        ignore_directories = True
#        case_sensitive = True
        
#        event_handler = PatternMatchingEventHandler(self.fileFilters, ignore_patterns, ignore_directories, case_sensitive)
#        event_handler.on_created = lambda *args, **kwargs: self.updateFiles()
#        event_handler.on_deleted = lambda *args, **kwargs: self.updateFiles()
#        event_handler.on_modified = lambda *args, **kwargs: self.updateFiles()
#        event_handler.on_moved = lambda *args, **kwargs: self.updateFiles()
#        
#        self.__observer.unschedule_all()
#        self.__observer.schedule(event_handler, self.__currentDir, recursive=False)
        
    def updateFiles(self):
        
        logger.debug("Load files from directory %s", self.__currentDir)
        
        #self.__filemodel.clear()
#        self.__filemodel.setColumnCount(2)
#        self.__filemodel.setHeaderData(0, qtc.Qt.Horizontal, 'File')
#        self.__filemodel.setHeaderData(1, qtc.Qt.Horizontal, 'Entry')
        
        Files = []
        to_add = []
        to_delete = []
        rows_to_delete = []
        
        filelst = qtc.QDir(self.__currentDir).entryList(self.fileFilters, qtc.QDir.Files | qtc.QDir.Readable)
        Nf = len(filelst)
        self.__prgBar.setMaximum(Nf+10)
        
        for i,f in enumerate(filelst):
            f = os.path.abspath(f)
            logger.debug(f)
            Files += [f,]
#                Files[f] = FileDatabase.getItemsFromFilename(f)
            if f not in self.__currfiles:
                to_add += [f]
                
            self.__prgBar.setValue(i+1)
                
        
        for f,v in self.__currfiles.items():
            if f not in Files and v is not None:
                to_delete += [f]
                
                for it in v:
                    rows_to_delete += [ it.row(), ]
                        
        rows_to_delete.sort(reverse=True)
        
        
        self.__prgBar.setValue(Nf+2)

        for f in to_delete:
            del self.__currfiles[f]
            
        self.__prgBar.setValue(Nf+3)
            
        for r in rows_to_delete:
            self.__filemodel.removeRow(r)
            
        self.__prgBar.setValue(Nf+4)
                
        for i,f in enumerate(to_add):
            try:
                fd = FileDatabase.getItemsFromFilename(f)
            except NotXPCSFileException: # Skip non-XPCS files
                self.__currfiles[f] = None
                continue
            except Exception as e: # This might come because processing is not finished.
                logger.info(f"Problem with {f} {e}")
                continue
                
            for uid, v in fd.items():
                it1 = qtg.QStandardItem(os.path.basename(v['fd'].filename))
                it2 = qtg.QStandardItem(v['key'])
                it3 = qtg.QStandardItem(v['fd'].title)
                
                its = [it1, it2, it3]
                
                for it in its:
                    it.setData(uid)
                    it.setEditable(False);
                
                self.__filemodel.appendRow( its )
                
                if f not in self.__currfiles:
                    self.__currfiles[f] = []
                
                self.__currfiles[f] += [it1,]
                
            v = int(np.ceil(6./len(to_add)*i))
            self.__prgBar.setValue(Nf+4+v)
        
        self.__prgBar.setValue(Nf+10)
        self.__filelist.header().setStretchLastSection(True)
        self.__filelist.resizeColumnToContents(0)
        
        logger.debug("files to add: \n %s"%"\n  ".join(to_add))
        logger.debug("files to delete: \n %s"%"\n  ".join(to_delete))
#        logger.debug("_currfiles state: \n %s"%"\n  ".join([f"{k} : {v}" for k,v in self.__currfiles.items()]))
        
        logger.debug("Files loaded")
            
    def onPlotButtonPressed(self):
        self.onPlotRequest.emit(self.filesSelected)
    
    @property
    def filesSelected(self):
        r = dict()
        
        for s in self.__filelist.selectedIndexes():
            it = self.__filemodel.itemFromIndex(s)
            
            dbe = FileDatabase.getDescriptorFromKey(it.data())
            logger.debug(dbe)
            
            fn = dbe['fd'].filename
            en = dbe['key']
            
            if fn not in r:
                r[fn] = []
                
            if en not in r[fn]:
                r[fn] += [en,]
        
        logger.debug(r)
            
        return r
        
        
        



# -*- coding: utf-8 -*-

import PyQt5.QtGui as qtg
import PyQt5.QtCore as qtc
import PyQt5.QtWidgets as qtw

import pandas as pd

from xpcsutilities.tools.result_file import XPCSResultFile

import numpy as np
import logging, os
logger = logging.getLogger(__name__)



class XPCSExportDialog(qtw.QDialog):
    """
    Contain the export dialog
    """
    
    def __init__(self, files, parent=None):
        super().__init__(parent)
        
        self.__files = files
        
        self.setWindowTitle("Export to ASCII")
        
        self.__layout = qtw.QFormLayout()
        self.setLayout(self.__layout)
        
        self.__exportext = qtw.QLineEdit()
        self.__exportext.setText("_ascii.txt")
        self.__layout.addRow("Filename extension", self.__exportext)
        
        self.__exportheaders = qtw.QCheckBox()
        self.__exportheaders.setChecked(True)
        self.__layout.addRow("Add headers to data export", self.__exportheaders)
        
        self.__exportpath = qtw.QPushButton(os.getcwd())
        self.__exportpath.clicked.connect(self.changeExportDirectory)
        self.__layout.addRow("Export to", self.__exportpath)
        
        self.__exportFmt = qtw.QComboBox()
        self.__exportFmt.addItem("Pandas DataFrame")
        self.__exportFmt.addItem("Array, one column per q and direction")
        self.__exportFmt.addItem("Array, one column per q, one section by direction")
        self.__layout.addRow("Export format", self.__exportFmt)
        
        self.__eraseStrategy = qtw.QComboBox()
        self.__eraseStrategy.addItem("Do nothing")
        self.__eraseStrategy.addItem("Overwrite")
        self.__eraseStrategy.addItem("Save new file with suffix")
        self.__layout.addRow("If output file already exist", self.__eraseStrategy)
        
        self.__exportbtn = qtw.QPushButton("Export")
        self.__exportbtn.clicked.connect(self.export)
        self.__layout.addRow(self.__exportbtn)
        
        
    def changeExportDirectory(self):
        """
        Change the directory where the data are exported
        """
        
        newd = qtw.QFileDialog.getExistingDirectory(self, "Export data to ...", self.__exportpath.text())
        
        if os.path.isdir(newd):
            self.__exportpath.setText(newd)
            
    def export(self):
        """
        Export the data files to ASCII
        """
        extension = self.__exportext.text() 
        basepath = self.__exportpath.text()
        for f in self.__files:
            
            logger.info(f"Exporting {f}")
            
            basename = os.path.basename(f)
            
            if '.' in basename:
                basename = '.'.join(basename.split('.')[:-1])
            
            filename = f"{basepath}/{basename}{extension}"
            
            logger.debug(f"Filename: {filename}")
            
            # If already exist, check what user wants
            if os.path.isfile(filename):
                # Option 1: skip the file
                if self.__eraseStrategy.currentIndex() == 0:
                    logger.info(f"Skip file {f}: Output file already exists")
                    continue
                
                # Option 2: Overwrite
                elif self.__eraseStrategy.currentIndex() == 1:
                    logger.info(f"Will overwrite output file {filename}")
                
                # Option 3: Find a free filename with suffix
                elif self.__eraseStrategy.currentIndex() == 2:
                
                    i = 0
                    newf = f"{filename}.{i}"
                    
                    while os.path.isfile(newf):
                        i += 1
                        newf = f"{filename}.{i}"
                        
                    filename = newf
                    logger.info(f"Found an alternative output filename: {filename}")
               
                logger.debug(f"Filename after overwrite policy: {filename}")
            
            with XPCSResultFile(f) as data_file:
                # Data format
                if self.__exportFmt.currentIndex() == 0: # Pandas format
                    data = []
                
                    for k in data_file.analysis_keys:
                        tau, cf, std = data_file.get_correlations(k)
                        qq = data_file.get_q_values(k)
                        
                        for i, q in enumerate(qq):
                            
                            dr = {'dir':k,
                                  'q': q,
                                  'tau': tau.ravel(),
                                  'cf': cf[i,:].ravel() }
                            
                            if std is not None:
                                dr['std'] = std[i,:].ravel()
                            
                            data.append(pd.DataFrame(dr))
            
                    fdata = pd.concat(data, ignore_index=True)
                    
                    fdata.to_csv(filename, float_format="%.5e", index=False)
                    
                elif self.__exportFmt.currentIndex() == 1: # Column/dir format
                
                    datas = None
                    titles = None
                    
                    for k in data_file.analysis_keys:
                        tau, cf, std = data_file.get_correlations(k)
                        qq = data_file.get_q_values(k)
                        
                        if datas is None:
                            datas = [tau.ravel(), ]
                            titles = ["lag", ]
                        
                        for i, q in enumerate(qq):
                            if std is not None:
                                datas += [cf[i,:].ravel(), std[i,:].ravel(),]
                                titles += [f"cf_{q:.2e}_{k}", f"std_{q:.2e}_{k}",]
                            else:
                                datas += [cf[i,:].ravel(),]
                                titles += [f"cf_{q:.2e}_{k}",]
                                
                            
                    datas = np.array(datas).T
                    
                    datas[~np.isfinite(datas)] = -10.
                            
                    with open(filename, 'w') as fd:
                        fd.write("#"+",".join(titles)+'\n')
                        np.savetxt(fd, datas, '%.5e', ',')
                    
                elif self.__exportFmt.currentIndex() == 2: # Section/dir format
                
                    with open(filename, 'w') as fd:
                        for k in data_file.analysis_keys:
                            datas = None
                            titles = None
                            tau, cf, std = data_file.get_correlations(k)
                            qq = data_file.get_q_values(k)
                            
                            if datas is None:
                                datas = [tau.ravel(), ]
                                titles = ["lag", ]
                            
                            for i, q in enumerate(qq):
                                if std is not None:
                                    datas += [cf[i,:].ravel(), std[i,:].ravel(),]
                                    titles += [f"cf_{q:.2e}", f"std_{q:.2e}",]
                                else:
                                    datas += [cf[i,:].ravel(),]
                                    titles += [f"cf_{q:.2e}",]
                                
                            datas = np.array(datas).T
                    
                            datas[~np.isfinite(datas)] = -10.
                            
                            fd.write(f"# {k}\n")
                            fd.write("#"+",".join(titles)+'\n')
                            np.savetxt(fd, datas, '%.5e', ',')
                            fd.write('\n\n')
                        
        self.close()
                





# -*- coding: utf-8 -*-



import numpy as np
import scipy.optimize as opt


def LT(s, G, tau):
    """
    Compute the discrete laplace transform (DLT) of G(s) over tau
    
    :param s: Laplace variable (s)
    :param G: Function to compute the DLT, defined on s
    :param tau: Coodinates where to evaluate the DLT (s)
    
    :return: The Discrete Laplace Transform of G
    """
    
    tau = np.array(tau).ravel()
    s = np.array(s).ravel()
    G = np.array(G).ravel()
    
    tau.shape = (tau.shape[0],1)
    s.shape = (s.shape[0],1)
    G.shape = (s.shape[0],1)
    
    return np.trapz(G*np.exp(-tau.T/s), x=s.ravel(), axis=0)
    
def g2LT(s, G, tau, beta=1):
    """
    Compute the intensity-intensity autocorrelation function from relaxation time density
    
    :param s: Laplace variable (s)
    :param G: Relaxation time density
    :param tau: Coordinates where to evaluate the g2
    :param beta: Speckles contrast
    
    :return: The Discrete Laplace Transform of G
    """
    return beta*(LT(s, G, tau) / LT(s, G, [0]))**2
    

def contin(tau, g2, s, gamma2=.2, beta0=None, G0=None):
    """
    Compute Inverse Laplace Transform (ILT) with CONTIN method
    
    :param tau: Lag time (s)
    :param g2: Intensity-Intensity autocorrelation function
    :param s: Laplace variable
    :param gamma2: 2nd derivative weight
    :param beta0: Initial value of beta
    :param G0: Initial values of G
    
    :return: (beta, ILT)
    """
    
    def fun(G, s, tau, g2, gamma2=.2):
        """
        The function to optimize
        """
        
        beta = G[0]
        G = G[1:]
        
        # Error
        err = np.sum((g2LT(s, G, tau, beta) - g2)**2) # I dont multiply by delta tau because I want geometric weighting with tau. With logarithmic tau spacing, this is division by Delta tau, so this cancel out.
        
        # Regularization function
        reg = np.trapz((np.diff(np.diff(G)))**2, x=s[1:-1])
        
        return err + gamma2*reg
    
    if beta0 is None:
        beta0 = 1
    
    if G0 is None:
        G0 = s**0/len(s)
    
    sol = opt.minimize(fun, [beta0, *G0], (s, tau, g2, gamma2), method='trust-constr', bounds=[(0,1),] + [(0, np.inf),]*len(s))
    
    return (sol.x[0], sol.x[1:])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Oct  7 15:46:18 2021

@author: opid02
"""

import h5py, hdf5plugin
import logging
import numpy as np
import pandas as pd
import os
from .result_file_old import XPCSResultFileOld

logger = logging.getLogger(__name__)

class NotXPCSFileException(Exception):
    pass


class XPCSResultFile(object):
    def __init__(self, filename):
        self.__filename = filename
        self._fd = None
    
    def __enter__(self):
        """
        Allow with statement
        """
        try:
            self._fd = XPCSResultFileNew(self.__filename)
        except NotXPCSFileException:
            pass
        
        if self._fd is None:
            self._fd = XPCSResultFileOld(self.__filename)
        
        return self._fd
    
    def __exit__(self, exc_type, exc_value, exc_tb):
        self._fd = None
        return False



class XPCSResultFileNew(object):
    def __init__(self, filename):
        """
        Initialize the class with filename, open file
        """
        logger.debug("Open file %s",filename)
        self.__filename = filename
        self.__fd = None
        self.__fd = h5py.File(self.__filename, 'r', locking=False)
        
        self.__analysis_path = "/entry_0000/correlations/"
        self.__headers = None
        self.__aheaders = None
        
        self.checkFile()
        
    def __del__(self):
        """
        Take care of closing the file
        """
        if self.__fd is not None:
            self.close()
        
    def close(self):
        logger.debug("Closing file %s", self.__filename)
        self.__fd.close()
        self.__fd = None
        
        
    def checkFile(self):
        """
        Check magic string and version
        """
        if "MagicString" in self.__fd.attrs and self.__fd.attrs['MagicString'] == "XPCSResultFile_v2.0":
            return
        
        raise NotXPCSFileException("Unknown MagicString.")
            
            
    @property
    def headers(self):
        if type(self.__headers) != dict:
            self.__headers = dict()
            
            for k in self.__fd['/entry_0000/parameters'].keys():
                v = self.__fd['/entry_0000/parameters/%s'%k][()]
                
                if isinstance(v, bytes):
                    v = v.decode()
                
                self.__headers[k] = v
            
        return self.__headers
    
    @property
    def acq_headers(self):
        if type(self.__aheaders) != dict:
            self.__aheaders = dict()
            
            for k in self.__fd['/entry_0000/acquisition'].keys():
                v = self.__fd['/entry_0000/acquisition/%s'%k][()]
                
                if isinstance(v, bytes):
                    v = v.decode()
                
                self.__aheaders[k] = v
            
        return self.__aheaders
    
    @property
    def title(self):
        if 'title' in self.headers:
            return self.headers['title']
        
        return ""
    
    @property
    def timestamp(self):
        if 'timestamp' in self.acq_headers:
            return int(self.acq_headers['timestamp'])
        
        return None
        
            
    @property
    def filename(self):
        return os.path.basename(self.__filename)
            
    @property
    def saxs_curve_q(self):
        """
        q vector of SAXS 1D reduced curve
        """
        if not hasattr(self, '_XPCSResultFile__1D_average_q'):
            logger.debug("Load 1D saxs curve average")
            self.__1D_average_q = np.array(self.__fd.get('entry_0000/time_averaged/azimuthal_average/q'))
            
        return self.__1D_average_q
    
    @property
    def saxs_curve_Iq(self):
        """
        I(q) vector of SAXS 1D reduced curve
        """
        if not hasattr(self, '_XPCSResultFile__1D_average_Iq'):
            logger.debug("Load 1D saxs curve average")
            self.__1D_average_Iq = np.array(self.__fd.get('entry_0000/time_averaged/azimuthal_average/Intensity')).ravel()
            
        return self.__1D_average_Iq
    
    @property
    def default_2D_pattern(self):
        return "frame_average_norm"
    
    @property
    def available_2D_patterns(self):
        """
        Return the available 2D patterns keys, and the default one
        """
        R = []
        
        grp = self.__fd['/entry_0000/time_averaged']
        
        for f in grp:
            if isinstance(grp[f], h5py.Dataset) and len(grp[f].shape) == 2:
                R += [f,]
        
        return R
    
    def get_2D_pattern(self, key):
        """
        Return the corrected time averaged 2D pattern
        """
        # Cache is performed by h5py
        return np.array(self.__fd.get(f'entry_0000/time_averaged/{key}'))
    
    @property
    def available_2D_parameters(self):
        """
        Return the available 2D patterns keys, and the default one
        """
        R = []
        
        grp = self.__fd['/entry_0000/parameters']
        
        for f in grp:
            if isinstance(grp[f], h5py.Dataset) and len(grp[f].shape) == 2:
                R += [f,]
        
        return R
    
    
    def get_2D_parameter(self, key):
        """
        Return the corrected time averaged 2D pattern
        """
        # Cache is performed by h5py
        return np.array(self.__fd.get(f'entry_0000/parameters/{key}'))
    
    
    @property
    def analysis_keys(self):
        """
        Return the analysis keys, to be passed in analysis getter functions
        """
        return [ *self.__fd[self.__analysis_path].keys() ]
    
    
    def get_qmask(self, key):
        """
        Return the q-mask for specific analysis
        """

        roi_masks = np.array(self.__fd.get(f'entry_0000/correlations/{key}/roi_masks'))
        
        R = np.full(roi_masks.shape[1:], 0, dtype=np.uint8)
        
        for i in range(roi_masks.shape[0]):
            R[roi_masks[i,:,:]] = i+1
            
        return R
    
    def get_roi_masks(self, key):
        """
        Return the roi mask for specific analysis
        """

        return np.array(self.__fd.get(f'entry_0000/correlations/{key}/roi_masks'))
    
    def get_correlations(self, key):
        """
        Return lag, correlation function and standard deviation if any from specific analysis
        """
            
        lag = np.array(self.__fd.get(f'entry_0000/correlations/{key}/g2/lag'))
        cf  = np.array(self.__fd.get(f'entry_0000/correlations/{key}/g2/cf'))
        
        if f'entry_0000/correlations/{key}/g2/err' in self.__fd:
            std = np.array(self.__fd.get(f'entry_0000/correlations/{key}/g2/err'))
        else:
            std = None
            
        return (lag, cf, std)
    
    def get_q_values(self, key):
        """
        Return the q-values for the specific analysis
        """
        
        return np.array(self.__fd.get(f'entry_0000/correlations/{key}/parameters/q'))    
    
    def get_ttcf(self, key, index):
        """
        Return the actual q indexes used for this specific analysis
        """
        
        lag = np.array(self.__fd.get(f'entry_0000/correlations/{key}/twotime/lag'))
        age = np.array(self.__fd.get(f'entry_0000/correlations/{key}/twotime/age'))
        cf  = np.array(self.__fd[f'entry_0000/correlations/{key}/twotime/ttcf'][index,:,:])
            
        return cf, lag, age
    
    def get_ttcfs(self, key):
        """
        Return the ttcf 3D matrix
        """
        
        lag = np.array(self.__fd.get(f'entry_0000/correlations/{key}/twotime/lag'))
        age = np.array(self.__fd.get(f'entry_0000/correlations/{key}/twotime/age'))
        cf  = np.array(self.__fd[f'entry_0000/correlations/{key}/twotime/ttcf'])
            
        return cf, lag, age
        




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Oct  7 15:46:18 2021

@author: opid02
"""

import h5py, hdf5plugin
import logging
import numpy as np
import pandas as pd
import os

logger = logging.getLogger(__name__)


class XPCSResultFileOld(object):
    def __init__(self, filename):
        """
        Initialize the class with filename, open file
        """
        logger.debug("Open file %s",filename)
        self.__filename = filename
        self.__fd = None
        self.__fd = h5py.File(self.__filename, 'r', locking=False)
        
        self.__analysis_path = "/entry_0000/dynamix/correlations/directions/"
        self.__analysis_basepath = self.__analysis_path+"/%s"

        self.__headers = None
        
        self.checkFile()
        
    def __del__(self):
        """
        Take care of closing the file
        """
        if self.__fd is not None:
            self.close()            
        
    def close(self):
        logger.debug("Closing file %s", self.__filename)
        self.__fd.close()
        
        
    def checkFile(self):
        """
        Check magic string and version
        """
        from .result_file import NotXPCSFileException
        if "entry_0000/title" in self.__fd and "version" in self.__fd["entry_0000"].attrs:
            self.__version = self.__fd['entry_0000'].attrs['version']
            if self.__fd.get('entry_0000/title')[0].decode('utf-8') != "XPCSResultFile":
                raise NotXPCSFileException("Invalid title. Should be XPCSResultFile, not %s"%self.__fd.get('entry_0000/title')[0].decode('utf-8'))
                
            logger.debug("File version: %s", self.__version)
        else:
            raise NotXPCSFileException("Can't find filetype or version for given file.")
            

            
    @property
    def headers(self):
        if type(self.__headers) != dict:
            self.__headers = dict()
            
            for k in self.__fd['/entry_0000/configuration'].keys():
                v = self.__fd['/entry_0000/configuration/%s'%k][()]
                
                if isinstance(v, bytes):
                    v = v.decode()
                
                self.__headers[k] = v
            
        return self.__headers
    
    @property
    def acq_headers(self):
        return {}
    
    @property
    def title(self):
        if 'title' in self.headers:
            return self.headers['title']
        
        return ""
    
    @property
    def timestamp(self):
        if 'timestamp' in self.__fd['/entry_0000/']:
            return self.__fd['/entry_0000/timestamp'][()]
        
        return None
        
            
    @property
    def filename(self):
        return os.path.basename(self.__filename)
            
    @property
    def saxs_curve_q(self):
        """
        q vector of SAXS 1D reduced curve
        """
        if not hasattr(self, '_XPCSResultFile__1D_average_q'):
            logger.debug("Load 1D saxs curve average")
            self.__1D_average_q = np.array(self.__fd.get('entry_0000/dynamix/time_averaged/1D/q'))
            
        return self.__1D_average_q
    
    @property
    def saxs_curve_Iq(self):
        """
        I(q) vector of SAXS 1D reduced curve
        """
        if not hasattr(self, '_XPCSResultFile__1D_average_Iq'):
            logger.debug("Load 1D saxs curve average")
            self.__1D_average_Iq = np.array(self.__fd.get('entry_0000/dynamix/time_averaged/1D/Iq')).ravel()
            
        return self.__1D_average_Iq
    
    @property
    def default_2D_pattern(self):
        return "corrected"
    
    @property
    def available_2D_patterns(self):
        """
        Return the available 2D patterns keys, and the default one
        """
        R = []
        
        grp = self.__fd['/entry_0000/dynamix/time_averaged']
        
        for f in grp:
            if isinstance(grp[f], h5py.Dataset) and len(grp[f].shape) == 2:
                R += [f,]
        
        return R
        
    
    def get_2D_pattern(self, key):
        """
        Return the corrected time averaged 2D pattern
        """
        # Cache is performed by h5py
        return np.array(self.__fd.get(f'entry_0000/dynamix/time_averaged/{key}'))
    
    @property
    def available_2D_parameters(self):
        """
        Return the available 2D patterns keys, and the default one
        """
        R = []
        
        grp = self.__fd['/entry_0000/dynamix/parameters']
        
        for f in grp:
            if isinstance(grp[f], h5py.Dataset) and len(grp[f].shape) == 2:
                R += [f,]
        
        return R
    
    def get_2D_parameter(self, key):
        """
        Return the corrected time averaged 2D pattern
        """
        # Cache is performed by h5py
        return np.array(self.__fd.get(f'entry_0000/dynamix/parameters/{key}'))
    
    
    @property
    def analysis_keys(self):
        """
        Return the analysis keys, to be passed in analysis getter functions
        """
        return [ *self.__fd[self.__analysis_path].keys() ]
    
    
    def get_qmask(self, key):
        """
        Return the q-mask for specific analysis
        """

        fkey = self.__analysis_basepath%key
        
        if fkey not in self.__fd:
            raise ValueError("Unknown analysis %s"%(key))

        return np.array(self.__fd.get('%s/qmask'%fkey))

    
    def get_roi_masks(self, key):
        """
        Return the roi mask for specific analysis
        """

        qm = self.get_qmask(key)
        M = int(np.nanmax(qm))
        
        roi_masks = np.empty((M-1, *qm.shape), dtype=np.bool)
        
        for i in range(M-1):
            roi_masks[i,:,:] = qm == i+1

        return roi_masks
    
    def get_correlations(self, key):
        """
        Return lag, correlation function and standard deviation if any from specific analysis
        """
        fkey = self.__analysis_basepath%key
        
        if fkey not in self.__fd:
            raise ValueError("Unknown analysis %s"%(key))
            
        ts = np.array(self.__fd.get('%s/correlation/timeshift'%fkey))
        
        if len(ts.shape) == 0:
            ts = np.array(self.__fd.get('%s/correlation/lag'%fkey))
        
        cf = np.array(self.__fd.get('%s/correlation/cf'%fkey))
        
        if '%s/correlation/std'%fkey in self.__fd:
            std = np.array(self.__fd.get('%s/correlation/std'%fkey))
        else:
            std = None
            
        return (ts.ravel(), cf.T, std.T)
    
    def _get_q_index(self, key):
        """
        Return the actual q indexes used for this specific analysis
        """
        fkey = self.__analysis_basepath%key
        
        if fkey not in self.__fd:
            raise ValueError("Unknown analysis %s"%(key))
            
        return np.array(self.__fd.get('%s/q_index'%fkey), dtype=np.int32)
    
    def get_q_values(self, key):
        """
        Return the q-values for the specific analysis
        """
        
        qidx = self._get_q_index(key)
        rng  = np.array(self.__fd.get('entry_0000/dynamix/correlations/q_index_min_max'))
        
        msk = np.isin(rng[:,0], qidx)
        
        return (rng[msk,1] + rng[msk,2])/2.
    
    def get_ttcf(self, key, index):
        """
        Return the actual q indexes used for this specific analysis
        """
        fkey = self.__analysis_basepath%key
        
        if fkey not in self.__fd:
            raise ValueError("Unknown analysis %s"%(key))
            
        ttcf_ds = self.__fd['%s/2times_correlation/ttcf'%fkey]
        
        if index < 1 or index > ttcf_ds.shape[0]:
            raise ValueError(f"Invalid index {index} {ttcf_ds.shape}")
            
        age = np.array(self.__fd.get('%s/2times_correlation/age'%fkey))
        timeshift = np.array(self.__fd.get('%s/2times_correlation/timeshift'%fkey))
        
        if len(timeshift.shape) == 0:
            timeshift = np.array(self.__fd.get('%s/2times_correlation/lag'%fkey))
        
        return (np.array(ttcf_ds[index-1,:,:], dtype=ttcf_ds.dtype), timeshift.ravel(), age.ravel())
    
    def get_ttcfs(self, key):
        """
        Return the ttcf 3D matrix
        """
        fkey = self.__analysis_basepath%key
        
        if fkey not in self.__fd:
            raise ValueError("Unknown analysis %s"%(key))
            
        ttcf_ds = self.__fd['%s/2times_correlation/ttcf'%fkey]
            
        age = np.array(self.__fd.get('%s/2times_correlation/age'%fkey))
        timeshift = np.array(self.__fd.get('%s/2times_correlation/timeshift'%fkey))
        
        if len(timeshift.shape) == 0:
            timeshift = np.array(self.__fd.get('%s/2times_correlation/lag'%fkey))
            
        return (np.array(ttcf_ds, dtype=ttcf_ds.dtype), timeshift.ravel(), age.ravel())

        




# -*- coding: utf-8 -*-

import matplotlib
matplotlib.use('Qt5Agg')

import PyQt5.QtWidgets as qtw
import PyQt5.QtCore as qtc

import xpcsutilities.tools.graphs as gt
from xpcsutilities.tools.result_file import XPCSResultFile
import numpy as np
import scipy.optimize as opt

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas, NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

import xpcsutilities.tools.common as cmm

import logging
logger = logging.getLogger(__name__)


class GraphSAXSWithQRange(qtw.QWidget):
    def __init__(self):
        super().__init__()
        
        #Layout
        self.__layout = qtw.QVBoxLayout()
        self.setLayout(self.__layout)
        
        #Toolbar, figures
        self.__canvas = CanvasSAXSWithQRange()
        self.__nav = NavigationToolbar(self.__canvas, self)
        
        # Add to layout
        self.__layout.addWidget(self.__nav)
        self.__layout.addWidget(self.__canvas)        
        
    def plot(self, files):
        for f, v in files.items():
            with XPCSResultFile(f) as fd:
                self.__canvas.plot(fd,v)
            
        self.__canvas.draw()
        
    def clear(self):
        self.__canvas.clear()
        

class CanvasSAXSWithQRange(FigureCanvas):
    def __init__(self, parent=None, width=10, height=10, dpi=cmm.default_dpi):
        self.__fig = Figure(figsize=(width, height), dpi=dpi, tight_layout=True)
        self.__axes = self.__fig.add_subplot(111)
        super().__init__(self.__fig)
        
    def plot(self, file, entries):
        
        q = file.saxs_curve_q
        I = file.saxs_curve_Iq
       
        self.__axes.plot(q, I, label=f"{file.title} {file.filename}")
        
        qmin, qmax = np.inf,0
        
        for e in entries:
            qs   = file.get_q_values(e)
            qmin = min(np.nanmin(qs), qmin)
            qmax = max(np.nanmax(qs), qmax)
        
        (cmapi, normi, mv) = gt.qindex_colors([qmin, qmax])
        
        
        qmap = file.get_2D_parameter('qmap') if 'qmap' in file.available_2D_parameters else file.get_2D_pattern('qmap')
        for e in entries:
            roi_masks = file.get_roi_masks(e)
            
            logger.debug(roi_masks.shape)
            
            q_ranges = np.zeros((roi_masks.shape[0], 4))
            sym = gt.symbol(self._symid)
            self._symid += 1
            labelset = False
            for i in range(roi_masks.shape[0]):
                qM = qmap[roi_masks[i,:,:]]
                
                #print(np.sum(roi_masks[i,:,:]))
                
                if len(qM) == 0:
                    continue
                
                qm = np.mean(qM)
                q_ranges[i,0] = qm
                q_ranges[i,1] = qm - np.min(qM)
                q_ranges[i,2] = np.max(qM) - qm
                q_ranges[i,3] = I[np.argmin(np.abs(q - qm))]
                
                self.__axes.errorbar(q_ranges[i,0],
                                     q_ranges[i,3],
                                     marker=sym,
                                     xerr=q_ranges[i:i+1,1:3].T,
                                     color=cmapi(normi(q_ranges[i,0])),
                                     label=f"{e} {file.filename}" if not labelset else None)
                
                labelset = True
                
        self.__axes.set_xscale('log')
        self.__axes.set_yscale('log')
        self.__axes.set_xlabel('$q (nm^{-1})$')
        self.__axes.set_ylabel('$I(q)$')
        
        self.__axes.legend(fontsize=cmm.default_legend_font_size)
        
    def clear(self):
        self._symid = 0
        self.__axes.cla()




# -*- coding: utf-8 -*-

import matplotlib
matplotlib.use('Qt5Agg')

import PyQt5.QtWidgets as qtw
import PyQt5.QtCore as qtc

import xpcsutilities.tools.graphs as gt
from xpcsutilities.tools.result_file import XPCSResultFile
import numpy as np
import scipy.optimize as opt

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas, NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure

import xpcsutilities.tools.common as cmm

import logging
logger = logging.getLogger(__name__)

from xpcsutilities.tools import contin
from . import tab_res_contin


class MainContin(qtw.QWidget):
    """
    Contains the main tab for CONTIN analysis
    """
    
    def __init__(self):
        """
        Widget initialization
        """
        
        super().__init__()
        
        self.__layout = qtw.QVBoxLayout()
        self.setLayout(self.__layout)
        
        self.__main = qtw.QSplitter()
        self.__layout.addWidget(self.__main)
        
        self.__mainsettings = qtw.QWidget()
        self.__mainsettings.setMaximumWidth(500)
        self.__mainsettingslayout = qtw.QVBoxLayout()
        self.__mainsettings.setLayout(self.__mainsettingslayout)
        self.__main.addWidget(self.__mainsettings)
        
        self.__grp1settings = qtw.QGroupBox("CONTIN settings")
        self.__grp1settingslayout = qtw.QFormLayout()
        self.__grp1settings.setLayout(self.__grp1settingslayout)
        self.__mainsettingslayout.addWidget(self.__grp1settings)
        
        self.__mintime = qtw.QLineEdit()
        self.__mintime.setText("0")
        self.__grp1settingslayout.addRow("Minimum of Laplace variable (0 for auto)", self.__mintime)
        
        self.__maxtime = qtw.QLineEdit()
        self.__maxtime.setText("0")
        self.__grp1settingslayout.addRow("Maximum of Laplace variable (0 for auto)", self.__maxtime)
        
        self.__Ndistrib = qtw.QSpinBox()
        self.__Ndistrib.setMinimum(1)
        self.__Ndistrib.setMaximum(100)
        self.__Ndistrib.setValue(20)
        self.__grp1settingslayout.addRow("Number of Laplace points", self.__Ndistrib)
        
        # Progress bar
        self.__grpprgbar = qtw.QGroupBox()
        self.__mainsettingslayout.addWidget(self.__grpprgbar)
        self.__prgbarlayout = qtw.QFormLayout()
        self.__grpprgbar.setLayout(self.__prgbarlayout)
        
        self.__fileprocessbar = qtw.QProgressBar()
        self.__qprocessbar = qtw.QProgressBar()
        self.__prgbarlayout.addRow("Files: ", self.__fileprocessbar)
        self.__prgbarlayout.addRow("q: ", self.__qprocessbar)

        # Results tabs
        self.__restab = qtw.QTabWidget()
        self.__main.addWidget(self.__restab)

        
    def clear(self):
        """
        Used to remotely clean the tab. Ignored here.
        """
        pass
    
    def plot(self, filelist):
        """
        Start the analysis
        """
        mins = float(self.__mintime.text())
        maxs = float(self.__maxtime.text())
        
        self.__fileprocessbar.setMaximum(len(filelist))
        self.__fileprocessbar.setMinimum(0)
        self.__fileprocessbar.setValue(0)
        
        for j, fn in enumerate(filelist):
            with XPCSResultFile(fn) as fd:
                for k in fd.analysis_keys:
                    qs = fd.get_q_values(k)
                    tau, cf, std = fd.get_correlations(k)
                    
                    self.__qprocessbar.setMaximum(len(qs))
                    self.__qprocessbar.setMinimum(0)
                    self.__qprocessbar.setValue(0)
                    
                    
                    if mins == 0:
                        min_s = np.min(tau)
                    else:
                        min_s = mins
                        
                    if maxs == 0:
                        max_s = np.max(tau)
                    else:
                        max_s = maxs
        
                    s = np.logspace(np.log10(min_s), np.log10(max_s), self.__Ndistrib.value())
                    
                    beta = qs*0
                    G = np.zeros((len(s), len(qs)))
                    
                    for i, q in enumerate(qs):
                        beta[i], G[:,i] = contin.contin(tau, cf[:,i]-1, s)
                        self.__qprocessbar.setValue(i+1)
                        
                    restab = tab_res_contin.ContinResult()
                    self.__restab.addTab(restab, f"{fn} {k}")
                    restab.plot(qs, tau, cf, s, beta, G)
                    
            self.__fileprocessbar.setValue(j+1)
        
        



# -*- coding: utf-8 -*-

import matplotlib
matplotlib.use('Qt5Agg')

import PyQt5.QtWidgets as qtw

import numpy as np
import logging
logger = logging.getLogger(__name__)

#from .paramwidget import paramwidget
from .parametertable import ParamsTable

class CurveFitWidget(qtw.QGroupBox):
    def __init__(self):
        super().__init__("Curve fitting")
        
        self.__curve_fit_layout = qtw.QFormLayout()
        self.setLayout(self.__curve_fit_layout)
        
        self.__fit_max_lag = qtw.QDoubleSpinBox()
        self.__fit_max_lag.setDecimals(6)
        self.__fit_max_lag.setValue(0)
        self.__fit_max_lag.setMaximum(np.inf)
        self.__curve_fit_layout.addRow("Maximum lag time (0: deactivated)", self.__fit_max_lag)
        
        self.__g1_library = dict()
        self.__g1_library['dif'] = {'Name': 'Diffusive', 'Fun': 'exp(-D*t*q^2)', 'g1': lambda t,q,D,**kwargs : np.exp(-D*t*q**2) }
        self.__g1_library['difr'] = {'Name': 'Diffusive (radius)', 'Fun': 'exp(-kB*T*t*q^2/(6*pi*eta*r))', 'g1': lambda t,q,T,eta,r,**kwargs : np.exp(-1.380e-23*T*t*(q*1e9)**2/(6*np.pi*eta*r)) }
        self.__g1_library['exp'] = {'Name': 'Exponential', 'Fun': ' exp(-Gamma*t)', 'g1': lambda t,q,Gamma,**kwargs : np.exp(-Gamma*t) }
        self.__g1_library['stexp'] = {'Name': 'Stretched Exponential', 'Fun': ' exp(-Gamma*t^n)', 'g1': lambda t,q,Gamma,n,**kwargs : np.exp(-Gamma*t**n) }
        self.__g1_library['conv'] = {'Name': 'Convection', 'Fun': ' 1 + alpha*(sinc(t*q*v)-1)', 'g1': lambda t,q,v,alpha,**kwargs : 1.+alpha*(np.sinc(t*q*v/np.pi)-1.) }
        
        self.__curve_fit_layout.addRow(qtw.QLabel("Select g_1 to fit curves"))    
        
        for k,g1 in self.__g1_library.items():
            chkb = qtw.QCheckBox()
            chkb.stateChanged.connect(self.__displayFitParameters)
            
            g1['chkb'] = chkb
            self.__curve_fit_layout.addRow('%s %s'%(g1['Name'], g1['Fun']), g1['chkb'])
            
        self.__paramsTable = ParamsTable()
        self.__curve_fit_layout.addRow(self.__paramsTable)
            
        #self.__params_fit_layout = qtw.QFormLayout()
        #self.__curve_fit_layout.addRow(self.__params_fit_layout)
            
        # Fitting parameters
#        self.__g1_params = dict()
#        for k, g1 in self.__g1_library.items():
#            for p in g1['g1'].__code__.co_varnames:
#                if p not in self.__g1_params and p not in ('t','q','kwargs'):
#                    self.__g1_params[p] = {'name':p, 'widget': None }
        
        self.__fitContrast = False
          
    def setFitContrast(self, fit):
        self.__fitContrast = fit
        self.__displayFitParameters()
        
    def getFitContrast(self):
        return self.__fitContrast
        
    def __displayFitParameters(self):
        params = ['beta'] if self.__fitContrast else [] 
        for k, g1 in self.__g1_library.items():
            if g1['chkb'].isChecked():
                for p in g1['g1'].__code__.co_varnames:
                    if p not in params and p not in ('t','q','kwargs'):
                        params += [p,]
                        
        self.__paramsTable.showVariables(params)
                        
#        for n,p in self.__g1_params.items():
#            if n not in params:
#                if p['widget'] is not None:
#                    self.__params_fit_layout.removeRow(p['widget'])
#                    p['widget'] = None
#            elif p['widget'] is None:
#                p['widget'] = paramwidget()
#                label = qtw.QLabel(p['name'])
#                label.setStyleSheet("font-weight: bold; color: blue");
#                self.__params_fit_layout.addRow(label,p['widget'])
                
                
    def strModel(self):
        m = ''
        for k, g1 in self.__g1_library.items():
            if g1['chkb'].isChecked():
                m += '(%s)'%g1['Fun']
        return m
    
    def maxlag(self):
        val = self.__fit_max_lag.value()
        return val if val > 0 else +np.inf

    def fitfunction(self):
        """
        Return the fitting functions and copy of internal parameters dict
        """
        
        fncs = []
        for k, g1 in self.__g1_library.items():
            if g1['chkb'].isChecked():                
                fncs += [g1['g1'],]
                
        # Get parameters options
        parval = self.__paramsTable.getVariables()
        
        # This function will be passed to curve_fit, take arbitrary number of parameters to fit, return the cf matrix
        def fnc(t,q,*args, scale=True, sepf=False):
            q = q.ravel()
            N = q.shape[0]
            
            ret = np.ones((t.shape[0], N), dtype=np.float32)
            ret2 = np.ones((t.shape[0], N*len(fncs)), dtype=np.float32)
            pars = self.args2params(N,parval,*args, scale=scale) # Rebuild parameters for each curve
                    
            for i in range(N):
                for j, fn in enumerate(fncs):
                    res = fn(t,q[i],**{k:v[i] for k,v in pars.items()}) # Unpack parameters from arrays
                    ret[:,i] *= res**2
                    ret2[:,i*len(fncs)+j] = res**2
                    
                if 'beta' in pars:
                    ret[:,i] = ret[:,i]*pars['beta'][i]
                    
            return ret if not sepf else (ret, ret2, len(fncs))
            
        return fnc, parval.copy()
    
    def initValues(self, N : int, *, scale=True):
        """
        Return the initial values array to be passed to fitting function.
        """
        
        vals = []
#        for k,p in self.__g1_params.items():
#            scale = scales[k] if k in scales and scales[k] > 0. else 1.
#            if p['widget'] is not None and not p['widget'].isFixed():
#                val = p['widget'].value()/scale
#                vals += [val,]*N if p['widget'].isMultipleValue() else [val,]
        
        for k,v in self.__paramsTable.getVariables().items():
            if v['isFixed']:
                vals += [v['value'],]
            else:
                scale = 1.
                if scale and v['isScaled']:
                    if v['scale'] != 0.:
                        scale = v['scale']
                    else:
                        logger.warning("Scale is 0 for variable %s. Scaling disabled for this variable."%k)
                        
                val = v['value']/scale
                vals += [val,]*N if v['isMultiple'] else [val,]
                    
        return np.array(vals)
    
    def bounds(self, N : int, *, scale=True):
        """
        Return the lower and upper bounds to be passed to fitting function.
        """
        
        ma,Ma = [],[]
#        if scales is None:
#            scales = dict()
#            
#        for k,p in self.__g1_params.items():
#            scale = scales[k] if k in scales and scales[k] > 0. else 1.
#            if p['widget'] is not None and not p['widget'].isFixed():
#                m += [p['widget'].minval()/scale,]*N if p['widget'].isMultipleValue() else [p['widget'].minval()/scale,]
#                M += [p['widget'].maxval()/scale if p['widget'].maxval() != 0 else +np.inf,]*N if p['widget'].isMultipleValue() else [p['widget'].maxval()/scale if p['widget'].maxval() != 0 else +np.inf,]
        
        for k,v in self.__paramsTable.getVariables().items():
            if v['isFixed']:
                if v['value'] > 0:
                    ma += [v['value']*0.999,]
                    Ma += [v['value']*1.001,]
                elif v['value'] < 0:
                    ma += [v['value']*1.001,]
                    Ma += [v['value']*.999,]
                else:
                    ma += [-.0001,]
                    Ma += [.0001,]
            else:
                scale = 1.
                if scale and v['isScaled']:
                    if v['scale'] != 0.:
                        scale = v['scale']
                    else:
                        logger.warning("Scale is 0 for variable %s. Scaling disabled for this variable."%k)
                        
                m = v['min']/scale
                ma += [m,]*N if v['isMultiple'] else [m,]
                
                M = v['max']/scale
                Ma += [M,]*N if v['isMultiple'] else [M,]
        
        return np.array(ma), np.array(Ma)
    
    def args2params(self, N, parval, *args, scale=True):
        """
        Convert the array of parameters to parameters dictionary containing arrays of values
        """
#        
#        if scales is None:
#            scales = dict()
#        
#        if parval is None:
#            params = []
#            fncs = []
#            for k, g1 in self.__g1_library.items():
#                if g1['chkb'].isChecked():
#                    for p in g1['g1'].__code__.co_varnames:
#                        if p not in params and p not in ('q','t','kwargs'):
#                            params += [p,]
#                    
#                    fncs += [g1['g1'],]
#        
#            parval = dict()
#            for k,p in self.__g1_params.items():
#                if k in params: # Should be instanciated if in params
#                    parval[k] = { 'value': p['widget'].value(),
#                                  'isFixed': p['widget'].isFixed(),
#                                  'isMultiple': p['widget'].isMultipleValue() }
        
        pars=dict()
        i=0
        
        for k,p in parval.items():
            factor = p['scale'] if scale and p['isScaled'] and p['scale'] != 0 else 1.
            if p['isFixed']:
                pars[k] = np.array([p['value'],]*N).ravel()
                i += 1
            elif p['isMultiple']:
                pars[k] = np.array([a*factor for a in args[i:i+N]]).ravel()
                i += N
            else:
                pars[k] = np.array([args[i]*factor,]*N).ravel()
                i += 1
                
        return pars






# -*- coding: utf-8 -*-


from xpcsutilities.tools.result_file import XPCSResultFile
import logging, os

logger = logging.getLogger(__name__)

def main():
    import argparse
    
    parser = argparse.ArgumentParser(description="Display XPCS result files")
    
    parser.add_argument('-v', dest='verbose', action='store_true', default=False, help='Verbose mode (minimum message level=debug)')
    parser.add_argument('--quiet', dest='quiet', action='store_true', default=False, help='Quiet mode (minimum message level=warning)')
    parser.add_argument('-f', dest='force', action='store_true', default=False, help='Allow erasing previous existing output file')
    parser.add_argument('-S', dest='skip', action='store_true', default=False, help='Skip if output file is already existing')
    parser.add_argument('data_file', action='append', nargs='+', help='Data files')
    
    args = parser.parse_args()
            
    # Set the log level of each package according to the defined verbosity
    loglevel = logging.INFO
    
    if args.verbose:
        loglevel = logging.DEBUG
    elif args.quiet:
        loglevel = logging.WARNING
        
    for name in logging.root.manager.loggerDict:
        if name.startswith('xpcsutilities'):
            logging.getLogger(name).setLevel(loglevel)
            
    for f in args.data_file[0]:
        
        if not f.endswith('h5'):
            continue
        
        outf = '.'.join(f.split('.')[:-1])+'.txt'
        
        if args.skip and os.path.isfile(outf):
            logger.info(f"Skipping {f}")
            continue
        
        try:
            logger.info(f"Processing {f}")
            fd = XPCSResultFile(f)            
            
            fd.save_toAscii(outf, args.force)
                
        except Exception as e:
            logger.fatal(f"Problem with {f} : {e}")



# -*- coding: utf-8 -*-

import matplotlib
matplotlib.use('Qt5Agg')

import PyQt5.QtWidgets as qtw
import PyQt5.QtCore as qtc

import xpcsutilities.tools.graphs as gt
from xpcsutilities.tools.result_file import XPCSResultFile
import numpy as np
import scipy.optimize as opt

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas, NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure


import xpcsutilities.tools.common as cmm

import logging
logger = logging.getLogger(__name__)


class GraphSAXSPattern(qtw.QWidget):
    def __init__(self):
        super().__init__()
        
        #Layout
        self.__layout = qtw.QVBoxLayout()
        self.setLayout(self.__layout)
        
        #Figure
        self.__canvas = CanvasSAXSPattern()
        self.__nav = NavigationToolbar(self.__canvas, self)
        
        # Add to layout
        self.__layout.addWidget(self.__nav)
        self.__layout.addWidget(self.__canvas)
        
        # Checkbox to display mask and qmask
        self.__optlayout = qtw.QHBoxLayout()
        self.__opt1layout = qtw.QFormLayout()
        self.__layout.addLayout(self.__optlayout)
        self.__optlayout.addLayout(self.__opt1layout)
        
        self.__checkShowMask = qtw.QCheckBox(self)
        self.__checkShowMask.setChecked(True)
        self.__opt1layout.addRow("Display mask", self.__checkShowMask)
        
        self.__checkShowQMask= qtw.QCheckBox(self)
        self.__checkShowQMask.setChecked(True)
        self.__opt1layout.addRow("Display qmask", self.__checkShowQMask)
        
        self.__plotSelector = qtw.QComboBox(self)
        
        self.__opt1layout.addRow("Plot", self.__plotSelector)
        
    
    def clear(self):
        self.__canvas.clear()
        
    def plot(self, files):
        
        N = 0
        for k,v in files.items():
            N += len(v)
        
        self.__canvas.setNPlots(N)
        
        i = 0
        for k,v in files.items():
            with XPCSResultFile(k) as fd:
                if i == 0:
                    _prev = self.__plotSelector.currentText()
                    self.__plotSelector.clear()
                    self.__plotSelector.addItems(fd.available_2D_patterns + fd.available_2D_parameters)
                    
                    logger.debug(_prev)
                    self.__plotSelector.setCurrentText(fd.default_2D_pattern if _prev == "" else _prev)
                    
                for d in v:
                    self.__canvas.plot(fd,i,d, self.__plotSelector.currentText(), self.__checkShowQMask.isChecked(), self.__checkShowMask.isChecked())
                    i += 1
            
        self.__canvas.draw()
        
        
class CanvasSAXSPattern(FigureCanvas):
    
    PLOT_CORRECTED  = 0
    PLOT_RAW        = 1
    PLOT_GAUSS      = 2
    PLOT_SATURATION = 3
    
    def __init__(self, parent=None, width=5, height=5, dpi=cmm.default_dpi):
        self.__fig = Figure(figsize=(width, height), dpi=dpi, tight_layout=True)
        super().__init__(self.__fig)
        
        self.setMaximumHeight(100000)
        self.setSizePolicy(qtw.QSizePolicy.Expanding, qtw.QSizePolicy.Expanding)
        
    def clear(self):
        self.__fig.clf()
        
    def _format_coord(self, x, y):
        
        xi = int(x)
        yi = int(y)
        
        R = f"x, y = ({xi}, {yi})"
        
        if xi >= 0 and xi < self._qmap.shape[1] and yi >=0 and yi < self._qmap.shape[0]:
            if self._tmap is not None:
                R += f" q; theta = ({self._qmap[yi,xi]:.01e}; {self._tmap[yi,xi]:.02f})"
            else:
                R += f" q = {self._qmap[yi, xi]:.01e}"
            
        return R
        
    
    def _get_cursor_data(self, event):
        
        xi, yi = int(event.xdata), int(event.ydata)
        
        value = self._values[yi, xi]
        rois  = np.where(self._roi_masks[:, yi, xi])[0]
        mask  = ~np.isnan(self._mask[yi, xi])
        
        masks = ""
        if mask:
            masks = "[ masked ]"
        elif len(rois) > 0:
            masks = ", ".join([f"{v:d}" for v in rois])
            masks = f"q_index = [ {masks} ]"
        return f"I={value:.03e} {masks}"
        
        
    def setNPlots(self, N):
        
        nCols = int(np.max([N**.5//1.22,1]))
        nRows = int(np.max([np.ceil(N/nCols),1]))
        
        gs = self.__fig.add_gridspec(nRows, nCols*2,width_ratios=[*[30, 1]*nCols])
        
        self.__axes = []
        
        for y in range(nRows):
            for x in range(nCols):
                self.__axes += [(self.__fig.add_subplot(gs[y,x*2]), self.__fig.add_subplot(gs[y,x*2+1])), ]
        
    def plot(self, file, ii, dirk, plot_sel, draw_q=True, draw_mask=True):
        
        ax = self.__axes[ii][0]
        ax.set_title(f"{file.title} {file.filename} {dirk}", fontsize=cmm.default_title_font_size)
        
        if dirk not in file.analysis_keys:
            return
        
        patt = None
        
        if plot_sel in file.available_2D_patterns:
            patt = file.get_2D_pattern(plot_sel)
        elif plot_sel in file.available_2D_parameters:
            patt = file.get_2D_parameter(plot_sel)
            
        if patt is None:
            logger.warning("Not available.")
            return
    
        cmap = mpl.cm.jet
        norm = mpl.colors.LogNorm()
        
        # qmask colorbar
        roi_masks = file.get_roi_masks(dirk)
        (cmapi, normi, mv) = gt.qindex_colors([0, roi_masks.shape[0]-1])
        
        cmapi.set_bad(alpha=0)
        
        ax.imshow(patt, cmap=cmap, norm=norm, interpolation='none')
        
        
        mmask = np.ones_like(patt, dtype=np.float32)
        mmask[file.get_2D_parameter('mask') == 1] = np.nan
        
        if draw_mask:
            normm = mpl.colors.Normalize(vmin=0, vmax=1)
            ax.imshow(mmask, cmap='gray', norm=normm, alpha=.6)
            
        roiN = np.zeros(roi_masks.shape[1:], dtype=np.float32)
        roiS = np.zeros(roi_masks.shape[1:], dtype=np.float32)
        if draw_q:
            for i in range(roi_masks.shape[0]):
                roiN[roi_masks[i]] += 1
                roiS[roi_masks[i]] += i
                
            ax.imshow(roiS/roiN, cmap=cmapi, norm=normi, alpha=1.-(0.6)**roiN)
            
        self._values = patt
        self._roi_masks = roi_masks
        self._mask = mmask
        self._qmap = file.get_2D_parameter('qmap') if 'qmap' in file.available_2D_parameters else file.get_2D_pattern('qmap')
        self._tmap = file.get_2D_parameter('thetamap') if 'thetamap' in file.available_2D_parameters else None 
        
        ax1 = self.__axes[ii][1]
        #ax2 = self.__axes[ii][2]
        
        self.__fig.colorbar(mpl.cm.ScalarMappable(norm=norm, cmap=cmap), cax=ax1, label=None)
        #self.__fig.colorbar(mpl.cm.ScalarMappable(norm=normi, cmap=cmapi), cax=ax2, label="q", format=mpl.ticker.FuncFormatter(lambda x,p: "%.3e"%(file.qs[np.where(x == q_ranges[:,0])])), ticks=file.q_ranges[::((file.q_ranges.shape[0]//8)+1),0])


        ax.format_coord = self._format_coord
        ax.format_cursor_data = lambda s: s
        ax.get_cursor_data = self._get_cursor_data
        ax.mouseover = True
        logger.debug(ax.mouseover)
        
#        while 'motion_notify_event' in self.callbacks.callbacks:
#            self.callbacks.callbacks.pop('motion_notify_event')
#            
#        self.mpl_connect('motion_notify_event', self.onHover)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Oct  5 17:12:34 2021

@author: opid02
"""

import sys
if sys.version_info < (3,6):
    sys.exit('Sorry, Python < 3.6 is not supported')

from setuptools import setup, find_packages

try:
    import pypandoc
    long_description = pypandoc.convert_file('README.md', 'rst')
except(IOError, ImportError):
    long_description = open('README.md').read()

setup(name='xpcsutilities',
      version='1.0.3',
      python_requires='>3.6',
      description='ESRF ID02 XPCSUtilities',
      long_description=long_description,
      author='William Chèvremont',
      author_email='william.chevremont@esrf.fr',
      url="https://gitlab.esrf.fr/id02/xpcsutilities",
      install_requires=[
          'numpy',
          'scipy',
          'h5py>=3.4',
          'hdf5plugin',
          'fabio',
          'watchdog',
          'PyQt5',
          'matplotlib',
          'pandas'
      ],
      packages=find_packages(include=['xpcsutilities', 'xpcsutilities.*']),
      entry_points={
                  'console_scripts': ['XPCSUtilities=xpcsutilities.bin.main:main',
                                    ]
              },
      package_data={'': ['xpcsutilities-logo.svg']},
      include_package_data=True,
    )




# -*- coding: utf-8 -*-

import PyQt5.QtWidgets as qtw
import numpy as np

class ParamsTable(qtw.QWidget):
    def __init__(self):
        super().__init__()
        
        self.__defaultValues = dict()
        
        self.__defaultValues['beta'] = dict()
        self.__defaultValues['beta']['init'] = .5
        self.__defaultValues['beta']['scale'] = 1
        self.__defaultValues['beta']['min'] = 0
        self.__defaultValues['beta']['max'] = 1
        self.__defaultValues['beta']['fixed'] = False
        self.__defaultValues['beta']['scaled'] = False
        self.__defaultValues['beta']['qdep'] = True
        
        self.__defaultValues['D'] = dict()
        self.__defaultValues['D']['init'] = 1e6
        self.__defaultValues['D']['scale'] = 1e6
        self.__defaultValues['D']['min'] = 0
        self.__defaultValues['D']['max'] = np.inf
        self.__defaultValues['D']['fixed'] = False
        self.__defaultValues['D']['scaled'] = False
        self.__defaultValues['D']['qdep'] = False
        
        self.__defaultValues['v'] = dict()
        self.__defaultValues['v']['init'] = 6e4
        self.__defaultValues['v']['scale'] = 1e4
        self.__defaultValues['v']['min'] = 0
        self.__defaultValues['v']['max'] = np.inf
        self.__defaultValues['v']['fixed'] = False
        self.__defaultValues['v']['scaled'] = False
        self.__defaultValues['v']['qdep'] = False
        
        self.__defaultValues['alpha'] = dict()
        self.__defaultValues['alpha']['init'] = .5
        self.__defaultValues['alpha']['scale'] = .1
        self.__defaultValues['alpha']['min'] = 0
        self.__defaultValues['alpha']['max'] = 1
        self.__defaultValues['alpha']['fixed'] = False
        self.__defaultValues['alpha']['scaled'] = False
        self.__defaultValues['alpha']['qdep'] = False
        
        self.__defaultValues['Gamma'] = dict()
        self.__defaultValues['Gamma']['init'] = 50
        self.__defaultValues['Gamma']['scale'] = 1e1
        self.__defaultValues['Gamma']['min'] = 0
        self.__defaultValues['Gamma']['max'] = np.inf
        self.__defaultValues['Gamma']['fixed'] = False
        self.__defaultValues['Gamma']['scaled'] = False
        self.__defaultValues['Gamma']['qdep'] = True
        
        self.__defaultValues['n'] = dict()
        self.__defaultValues['n']['init'] = 1
        self.__defaultValues['n']['scale'] = 1
        self.__defaultValues['n']['min'] = 1
        self.__defaultValues['n']['max'] = 2
        self.__defaultValues['n']['fixed'] = False
        self.__defaultValues['n']['scaled'] = False
        self.__defaultValues['n']['qdep'] = False
        
        self.__defaultValues['T'] = dict()
        self.__defaultValues['T']['init'] = 295.15
        self.__defaultValues['T']['scale'] = 1
        self.__defaultValues['T']['min'] = 0
        self.__defaultValues['T']['max'] = np.inf
        self.__defaultValues['T']['fixed'] = True
        self.__defaultValues['T']['scaled'] = False
        self.__defaultValues['T']['qdep'] = False
        
        self.__defaultValues['eta'] = dict()
        self.__defaultValues['eta']['init'] = .954e-3
        self.__defaultValues['eta']['scale'] = 1e-3
        self.__defaultValues['eta']['min'] = 0
        self.__defaultValues['eta']['max'] = np.inf
        self.__defaultValues['eta']['fixed'] = True
        self.__defaultValues['eta']['scaled'] = False
        self.__defaultValues['eta']['qdep'] = False
        
        self.__defaultValues['r'] = dict()
        self.__defaultValues['r']['init'] = 80e-9
        self.__defaultValues['r']['scale'] = 1e-9
        self.__defaultValues['r']['min'] = 0
        self.__defaultValues['r']['max'] = np.inf
        self.__defaultValues['r']['fixed'] = False
        self.__defaultValues['r']['scaled'] = False
        self.__defaultValues['r']['qdep'] = False
        
        self.__layout = qtw.QVBoxLayout()
        self.setLayout(self.__layout)
        
        self.__maintable = qtw.QTableWidget()
        self.__layout.addWidget(self.__maintable)
        
        self.__maintable.setColumnCount(1)
        self.__maintable.setRowCount(7)
        
        self.__titles = dict()
        self.__titles['initv'] = 'Initial'
        self.__titles['maxv'] = 'Maximum'
        self.__titles['minv'] = 'Minimum'
        self.__titles['fixed'] = 'Fixed'
        self.__titles['scaled'] = 'Scaled'
        self.__titles['scale'] = 'Scale'
        self.__titles['qdep'] = 'q-dep'
        
        for r,t in enumerate(self.__titles.values()):
            self.__maintable.setItem(r,0,qtw.QTableWidgetItem(t))
            
        self.__maintable.setColumnWidth(0, 80)
        self.__usedVariables = dict()
        
    def showVariables(self, lstv):
        
        for i in range(len(self.__usedVariables)):
            self.__maintable.hideColumn(i+1)
        
        for v in lstv:
            if v in self.__usedVariables:
                self.__maintable.showColumn(self.__usedVariables[v]['colid'])
            else:
                vard = dict()
                idc = self.__maintable.columnCount()
                self.__maintable.setColumnCount(idc+1)
                
                vard['colid'] = idc
                
                vard['initv'] = qtw.QLineEdit()
                vard['maxv'] = qtw.QLineEdit()
                vard['minv'] = qtw.QLineEdit()
                vard['fixed'] = qtw.QCheckBox()
                vard['scaled'] = qtw.QCheckBox()
                vard['scale'] = qtw.QLineEdit()
                vard['qdep'] = qtw.QCheckBox()
                
                for r,k in enumerate(self.__titles.keys()):
                    self.__maintable.setCellWidget(r,idc, vard[k])
                    
                self.__usedVariables[v] = vard
                self.__maintable.setColumnWidth(idc, 80)
                
                if v in self.__defaultValues:
                    dv = self.__defaultValues[v]
                    vard['initv'].setText('%.2e'%dv['init'])
                    vard['maxv'].setText('%.2e'%dv['max'])
                    vard['minv'].setText('%.2e'%dv['min'])
                    vard['scale'].setText('%.2e'%dv['scale'])
                    vard['scaled'].setChecked(dv['scaled'])
                    vard['fixed'].setChecked(dv['fixed'])
                    vard['qdep'].setChecked(dv['qdep'])
                else:
                    vard['initv'].setText('1.')
                    vard['maxv'].setText('inf')
                    vard['minv'].setText('0')
                    vard['scale'].setText('1')
                    vard['scaled'].setChecked(False)
                    vard['fixed'].setChecked(False)
                    vard['qdep'].setChecked(True)
                    
                    
        self.__maintable.setHorizontalHeaderLabels(['', *self.__usedVariables.keys()])
        
    def getVariables(self):
        
        ret = dict()
        
        for k,it in self.__usedVariables.items():
            if not self.__maintable.isColumnHidden(it['colid']):

                v = dict()
                v['value'] = float(it['initv'].text())
                v['min']  = float(it['minv'].text())
                v['max']  = float(it['maxv'].text())
                v['scale'] = float(it['scale'].text())
                
                v['isFixed'] = it['fixed'].isChecked()
                v['isScaled'] = it['scaled'].isChecked()
                v['isMultiple'] = it['qdep'].isChecked()
                
                ret[k] = v
                
        return ret
                
                
                
                
                
                
                
                
                
                
                
                
                
                
                
                
                
                